///////////////////////////////////////////////////////////////////////////
// Filename : monitor_decoder_ctl.h
// Project  : MP3 Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision : Jun.,2000
// Comment  : Waveform Monitor
//
//
#ifndef MONITOR_DECODER_CTL_INC
#define MONITOR_DECODER_CTL_INC

#include "systemc.h"

SC_MODULE(monitor_decoder_ctl)
{
    sc_in_clk           clk;                // Clock
    sc_in<sc_bit>       rst;                // Reset
    sc_in<sc_bit>       hdr_ready;          // Header Ready
    sc_in<sc_bit>       hdr_start;          // Header Start
    sc_in<sc_bit>       scl_ready;          // Scale Factor Ready
    sc_in<sc_bit>       scl_start;          // Scale Factor Start
    sc_in<sc_bit>       huff_ready;         // Huffman Ready
    sc_in<sc_bit>       huff_start;         // Huffman Start
    sc_in<sc_bit>       deq_ready;          // Dequantizer Ready
    sc_in<sc_bit>       deq_start;          // Dequantizer Start
    sc_in<sc_bit>       stereo_ready;       // Stereo Ready
    sc_in<sc_bit>       stereo_start;       // Stereo Start
    sc_in<sc_bit>       reorder_ready;      // Reorder Ready
    sc_in<sc_bit>       reorder_start;      // Reorder Start
    sc_in<sc_bit>       antialias_ready;    // Antialias Ready
    sc_in<sc_bit>       antialias_start;    // Antialias Start
    sc_in<sc_bit>       hybrid_ready;       // Hybrid Filter Bank Ready
    sc_in<sc_bit>       hybrid_start;       // Hybrid Filter Bank Start
    sc_in<sc_bit>       subband_ready;      // Subband Synthesis Filter Ready
    sc_in<sc_bit>       subband_start;      // Subband Synthesis Filter Start
    sc_in<sc_bit>       pcmout_start;
    sc_in<sc_bit>       stereo;
    sc_in<sc_bv<2> >    mode;
    sc_in<sc_bit>       version;
    sc_in<sc_bv<2> >    block_type;
    sc_in<sc_bit>       channel;
    sc_in<sc_bit>       granule;
    sc_in<sc_bit>       m_init_done;        // Frame FIFO Initializer
    sc_in<sc_bit>       mem_init;
    sc_in<sc_bit>       stream_full;        // Frame FIFO Fill
    sc_in<sc_bit>       fill_stream;
    sc_in<sc_bit>       stop;               // STOP Button

    // Process ///////////////////////////////////////////////////////////
    void proc_monitor();

    // Signals to be taced ///////////////////////////////////////////////
    bool    t_clk;
    bool    t_rst;

    bool    t_hdr_ready;        // Header Ready
    bool    t_hdr_start;        // Header Start

    bool    t_scl_ready;        // Scale Factor Ready
    bool    t_scl_start;        // Scale Factor Start

    bool    t_huff_ready;       // Huffman Ready
    bool    t_huff_start;       // Huffman Start

    bool    t_deq_ready;        // Dequantizer Ready
    bool    t_deq_start;        // Dequantizer Start

    bool    t_stereo_ready;     // Stereo Ready
    bool    t_stereo_start;     // Stereo Start

    bool    t_reorder_ready;    // Reorder Ready
    bool    t_reorder_start;    // Reorder Start

    bool    t_antialias_ready;  // Antialias Ready
    bool    t_antialias_start;  // Antialias Start

    bool    t_hybrid_ready;     // Hybrid Filter Bank Ready
    bool    t_hybrid_start;     // Hybrid Filter Bank Start

    bool    t_subband_ready;    // Subband Synthesis Filter Ready
    bool    t_subband_start;    // Subband Synthesis Filter Start

    bool    t_pcmout_start;

    bool    t_granule;
    bool    t_channel;

    bool    t_m_init_done;
    bool    t_mem_init;

    bool    t_stream_full;
    bool    t_fill_stream;

    bool    t_stop;

    // Waveform //////////////////////////////////////////////////////////////
    public:
        sc_trace_file*  tf;

    // Constructor ///////////////////////////////////////////////////////////
    SC_CTOR(monitor_decoder_ctl)
    {
        SC_METHOD(proc_monitor);
        sensitive
            << clk << rst           // Clock & Reset
            << hdr_ready
            << hdr_start            // Header
            << scl_ready
            << scl_start            // Scale Factor
            << huff_ready
            << huff_start           // Huffman
            << deq_ready
            << deq_start            // Dequantizer
            << stereo_ready
            << stereo_start         // Stereo
            << reorder_ready
            << reorder_start        // Reorder
            << antialias_ready
            << antialias_start      // Antialias
            << hybrid_ready
            << hybrid_start         // Hybrid Filter Bank
            << subband_ready
            << subband_start        // Subband Synthesis Filter
            << pcmout_start
            << granule
            << channel
            << m_init_done
            << mem_init
            << stream_full
            << fill_stream
            << stop;

        // Waaveform ///////////////////////////////
        tf = sc_create_vcd_trace_file("decoder_ctl");
        ((vcd_trace_file*)tf)->sc_set_vcd_time_unit(-9);

        sc_trace(tf, t_clk, "clk");
        sc_trace(tf, t_rst, "rst");
        sc_trace(tf, t_hdr_ready, "hdr_ready");
        sc_trace(tf, t_hdr_start, "hdr_start");
        sc_trace(tf, t_scl_ready, "scl_ready");
        sc_trace(tf, t_scl_start, "scl_start");
        sc_trace(tf, t_huff_ready, "huff_ready");
        sc_trace(tf, t_huff_start, "huff_start");
        sc_trace(tf, t_deq_ready, "deq_ready");
        sc_trace(tf, t_deq_start, "deq_start");
        sc_trace(tf, t_stereo_ready, "stereo_ready");
        sc_trace(tf, t_stereo_start, "stereo_start");
        sc_trace(tf, t_reorder_ready, "reorder_ready");
        sc_trace(tf, t_reorder_start, "reorder_start");
        sc_trace(tf, t_antialias_ready, "antialias_ready");
        sc_trace(tf, t_antialias_start, "antialias_start");
        sc_trace(tf, t_hybrid_ready, "hybrid_ready");
        sc_trace(tf, t_hybrid_start, "hybrid_start");
        sc_trace(tf, t_subband_ready, "subband_ready");
        sc_trace(tf, t_subband_start, "subband_start");
        sc_trace(tf, t_pcmout_start, "pcmout_start");
        sc_trace(tf, t_channel, "channel");
        sc_trace(tf, t_granule, "granule");
        sc_trace(tf, t_m_init_done, "m_init_done");
        sc_trace(tf, t_mem_init, "mem_init");
        sc_trace(tf, t_stream_full, "stream_full");
        sc_trace(tf, t_fill_stream, "fill_stream");
        sc_trace(tf, t_stop, "stop");
    }
};

#endif
