///////////////////////////////////////////////////////////////////////////
// Filename : monitor_mp3.h
// Project  : MP3 Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision : Jun.,2000
// Comment  : Waveform Monitor
//
//
#ifndef MONITOR_MP3_INC
#define MONITOR_MP3_INC

#include "systemc.h"

SC_MODULE(monitor_mp3)
{
    sc_in_clk           clk;                // Clock
    sc_in<sc_bit>       rst;                // Reset
    sc_in<sc_bit>       stop;
    sc_in<sc_bit>       mp3_req;
    sc_in<sc_bit>       mp3_ack;
    sc_in<sc_bv<8> >    mp3_data;
    sc_in<int>          mp3_count;

    // Process ///////////////////////////////////////////////////////////
    void proc_monitor();

    // Signals to be taced ///////////////////////////////////////////////
//  bool    t_clk;
//  bool    t_rst;
    bool    t_stop;
    bool    t_mp3_req;
    bool    t_mp3_ack;
    int     t_mp3_data;
    int     t_mp3_count;

    // Waveform //////////////////////////////////////////////////////////////
    public:
        sc_trace_file*  tf;

    // Constructor ///////////////////////////////////////////////////////////
    SC_CTOR(monitor_mp3)
    {
        SC_METHOD(proc_monitor);
        sensitive
//          << clk << rst                       // Clock & Reset
            << stop
            << mp3_req << mp3_ack << mp3_data;

        // Waaveform ///////////////////////////////
        tf = sc_create_vcd_trace_file("mp3_decoder");
        ((vcd_trace_file*)tf)->sc_set_vcd_time_unit(-9);

//      sc_trace(tf, t_clk, "clk");
//      sc_trace(tf, t_rst, "rst");
        sc_trace(tf, t_stop, "stop");
        sc_trace(tf, t_mp3_req, "mp3_req");
        sc_trace(tf, t_mp3_ack, "mp3_ack");
        sc_trace(tf, t_mp3_data, "mp3_data");
        sc_trace(tf, t_mp3_count, "mp3_count");
    }
};

#endif
