///////////////////////////////////////////////////////////////////////////
// Filename : sc_III_Antialias.cpp
// Project  : MP3 Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision :
// Comment  :
//
//

#include "MP3_Decoder.h"
#include "sc_III_Antialias.h"

#include "sc_III_Decoder_Mem.h"

void iii_antialias::proc_antialias()
{
    int     t_window_switching_flag = window_switching_flag.read();
    int     t_mixed_block_flag      = mixed_block_flag.read();
    int     t_block_type            = (sc_uint<2>)block_type.read();
    int     t_ch                    = channel.read();

    if (rst.read()==(sc_bit)'0')    // Async. Reset
    {
        Antialias_State = S_ANTIALIAS_WAIT;
        ready = (sc_bit)'1';
    }
    else if (clk.event() && clk.read()==(sc_bit)'1')    // Clocking Rising-Edge
    {
        switch(Antialias_State.read())
        {
            case S_ANTIALIAS_WAIT:
                if (start.read()==(sc_bit)'1')
                {
                    ready = (sc_bit)'0';
                    Antialias_State = S_ANTIALIAS_DECODE;
                }
                break;

            case S_ANTIALIAS_DECODE:
                if (start.read()==(sc_bit)'0')
                {
                    III_antialias(
                        t_window_switching_flag,
                        t_block_type,
                        t_mixed_block_flag,
                        t_ch );

                    Antialias_State = S_ANTIALIAS_DOWNLOAD;
                }
                break;

            case S_ANTIALIAS_DOWNLOAD:
                Antialias_State = S_ANTIALIAS_BUSY;
#ifdef _DEBUG_OUT
                {
                    FILE *fp;
                    int ss, sb;

                    fp = fopen("dbg_antialias.txt", "a+");

                    fprintf(fp, "Frame=%d/Granule=%d/Channel=%d\n",
                                    u_mp3_decoder.u_decoder_ctl->nFrame,
                                    ((u_mp3_decoder.u_decoder_ctl->gr)? 1:0),
                                    ((u_mp3_decoder.u_decoder_ctl->ch)? 1:0) );

                    for (sb=0; sb<SBLIMIT; sb++)
                        for (ss=0; ss<SSLIMIT; ss++)
                            fprintf(fp, "[%d][%2d][%2d] %16.12f\n", t_ch, sb, ss,
                                        u_mp3_decoder.u_decoder_mem->get_hybridIn(t_ch, sb, ss));
                    fflush(fp);
                    fclose(fp);
                }
#endif
                break;

            case S_ANTIALIAS_BUSY:
                ready = (sc_bit)'1';
                Antialias_State = S_ANTIALIAS_WAIT;
                break;

            default:
                Antialias_State = S_ANTIALIAS_WAIT;
                break;
        }
    }
}

///////////////////////////////////////////////////////////////////////////
// Antialias
//
void iii_antialias::III_antialias(
    int window_switching_flag,
    int block_type,
    int mixed_block_flag,
    int ch )
{
    double  bu, bd;  // upper and lower butterfly inputs
    int     ss, sb, sblim;

    // clear all inputs
//  for(sb=0;sb<SBLIMIT;sb++)
//      for(ss=0;ss<SSLIMIT;ss++)
//          u_mp3_decoder.u_decoder_mem->put_hybridIn( 
//                  u_mp3_decoder.u_decoder_mem->get_re( ch, sb, ss), ch, sb, ss);

    if (window_switching_flag && (block_type == 2) && !mixed_block_flag )
        return;

    if (window_switching_flag && (block_type == 2) && mixed_block_flag )
        sblim = 1;
    else
        sblim = SBLIMIT-1;

    // 31 alias-reduction operations between each pair of sub-bands
    // with 8 butterflies between each pair

    for(sb=0;sb<sblim;sb++)
        for(ss=0;ss<8;ss++)
        {
            bu = u_mp3_decoder.u_decoder_mem->get_re( ch, sb, 17-ss);
            bd = u_mp3_decoder.u_decoder_mem->get_re( ch, sb+1, ss);
            u_mp3_decoder.u_decoder_mem->put_hybridIn(
                    ((bu * cs[ss]) - (bd * ca[ss])), ch, sb, (17-ss) );
            u_mp3_decoder.u_decoder_mem->put_hybridIn(
                    ((bd * cs[ss]) + (bu * ca[ss])), ch, (sb+1), ss);
        }  
}
