///////////////////////////////////////////////////////////////////////////
// Filename : sc_III_Antialias.h
// Project  : MP3 Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision :
// Comment  :
//
//
#ifndef III_ANTIALIAS_INC
#define III_ANTIALIAS_INC

#include "systemc.h"

enum tAntialias_State {
    S_ANTIALIAS_WAIT,
    S_ANTIALIAS_DECODE,
    S_ANTIALIAS_DOWNLOAD,
    S_ANTIALIAS_BUSY };

SC_MODULE(iii_antialias)
{
    sc_in_clk           clk;    // Clock
    sc_in<sc_bit>       rst;    // Reset

    sc_in<sc_bit>       start;  // Antialias Start
    sc_out<sc_bit>      ready;  // Antialias Ready
    
    sc_in<sc_bit>       mixed_block_flag;
    sc_in<sc_bit>       window_switching_flag;
    sc_in<sc_bv<2> >    block_type;
    sc_in<sc_bit>       channel;

    // Process ///////////////////////////////////////////////////////////
    void proc_antialias();

    // Global Signal /////////////////////////////////////////////////////
    sc_signal<tAntialias_State> Antialias_State;    // FSM State Variable

    // Functions ////////////////////////////////////////////////////////
    void III_antialias(
        int window_switching_flag,
        int block_type,
        int mixed_block_flag,
        int ch );

    // Coefficients /////////////////////////////////////////////////////
    double  ca[8], cs[8];

    // Constructor ///////////////////////////////////////////////////////////
    SC_CTOR(iii_antialias)
    {
        SC_METHOD(proc_antialias);
        sensitive << clk << rst;

        // Initializer /////////////////////////
        {
            double  Ci[8] = {-0.6,-0.535,-0.33,-0.185,-0.095,-0.041,-0.0142,-0.0037};
            int     i;
            double  sq;

            for (i=0;i<8;i++)
            {
                sq=sqrt(1.0+Ci[i]*Ci[i]);
                cs[i] = 1.0/sq;
                ca[i] = Ci[i]/sq;
            }
        }
    }
};

#endif
