///////////////////////////////////////////////////////////////////////////
// Filename : sc_III_Common.h
// Project  : MP3 Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision :
// Comment  : Common Data Types for MP3
//
//
#ifndef MP3_COMMON_H
#define MP3_COMMON_H

#define PI  3.14159265358979

#define FRAME_SIZE  1024
#define BUFFER_SIZE 4096

#define MIN(A, B)   ((A) < (B) ? (A) : (B))
#define MAX(A, B)   ((A) > (B) ? (A) : (B))

// Global Definition /////////////////////////////////////////
#define SBLIMIT                 32
#define SSLIMIT                 18

/* MPEG Header Definitions - Mode Values */
#define MPG_MD_STEREO           0
#define MPG_MD_JOINT_STEREO     1
#define MPG_MD_DUAL_CHANNEL     2
#define MPG_MD_MONO             3

/* MPEG Header Definitions - ID Bit Values */
#define MPEG_AUDIO_ID           1
#define MPEG_PHASE2_LSF         0

// Global Type Definitions ////////////////////////////////////

/* Header Information Structure */
typedef struct {
    int version;
    int lay;
    int error_protection;
    int bitrate_index;
    int sampling_frequency;
    int padding;
    int extension;
    int mode;
    int mode_ext;
    int copyright;
    int original;
    int emphasis;
} layer, *the_layer;

/* Parent Structure Interpreting some Frame Parameters in Header */
typedef struct {
    layer       *header;        /* raw header information */
    int         actual_mode;    /* when writing IS, may forget if 0 chs */
    int         tab_num;        /* number of table as loaded */
    int         stereo;         /* 1 for mono, 2 for stereo */
    int         jsbound;        /* first band of joint stereo coding */
    int         sblimit;        /* total number of sub bands */
} frame_params;

/* Layer III side information. */
typedef struct {
    unsigned part2_3_length;
    unsigned big_values;
    unsigned global_gain;
    unsigned scalefac_compress;
    unsigned window_switching_flag;
    unsigned block_type;
    unsigned mixed_block_flag;
    unsigned table_select[3];
    unsigned subblock_gain[3];
    unsigned region0_count;
    unsigned region1_count;
    unsigned preflag;
    unsigned scalefac_scale;
    unsigned count1table_select;
} my_gr_info ;

typedef struct {
    unsigned main_data_begin;
    unsigned private_bits;
    struct {
        unsigned scfsi[4];
        struct gr_info_s {
            unsigned part2_3_length;
            unsigned big_values;
            unsigned global_gain;
            unsigned scalefac_compress;
            unsigned window_switching_flag;
            unsigned block_type;
            unsigned mixed_block_flag;
            unsigned table_select[3];
            unsigned subblock_gain[3];
            unsigned region0_count;
            unsigned region1_count;
            unsigned preflag;
            unsigned scalefac_scale;
            unsigned count1table_select;
            } gr[2];
        } ch[2];
    } III_side_info_t;

/* Layer III scale factors. */
typedef struct {
    int l[23];            /* [cb] */
    int s[3][13];         /* [window][cb] */
    } III_scalefac_t[2];  /* [ch] */

#endif
