///////////////////////////////////////////////////////////////////////////
// Filename : sc_III_Decoder_Mem.h
// Project  : Ans_MP3_Player/MP3_Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision :
// Comment  : Frame Buffer
//
//

#ifndef DECODER_MEM_INC
#define DECODER_MEM_INC

#include "systemc.h"
#include "sc_III_Common.h"

enum S_Mem_Init_Type {
    S_MEM_INIT_WAIT,
    S_MEM_INIT_BUSY,
    S_MEM_INIT_DONE
};

SC_MODULE(decoder_mem)
{
    sc_in_clk           clk;                // Clock
    sc_in<sc_bit>       rst;                // Reset

    sc_in<sc_bit>       mem_init;           // Frame Memory Initialize
    sc_out<sc_bit>      m_init_done;

    // Process ///////////////////////////////////////////////////////////
    void proc_mem_init();

    // Signals ///////////////////////////////////////////////////////////
    sc_signal<S_Mem_Init_Type>  Mem_Init_State;

    // Global Memories ///////////////////////////////////////////////////
    III_scalefac_t  scalefac;                           // Scale Factor

    double          ram[2][SBLIMIT][SSLIMIT];           // Decoder Memory
    double          prevblck[2][SBLIMIT][SSLIMIT];      // Hybrid result of previous Frame/Granule
    double          rawout[36];                         // Temp.
    short           pcm_sample[2][SSLIMIT][SBLIMIT];    // PCM output

    // Member functions //////////////////////////////////////////////////
    public:
        // Read/Write Scale factor data
        long    get_scalefac(int ch, int window, int cb, int block_type);
        void    put_scalefac(long int data, int ch, int window, int cb, int block_type);
        // Read/Write Huffman decoded
        long    get_is(int ch, int sb, int ss);
        void    put_is(long data, int ch, int sb, int ss);
        // Read/Write Requantized data
        double  get_ro(int ch, int sb, int ss);
        void    put_ro(double data, int ch, int sb, int ss);
        // Read/Write Stero processed data
        double  get_lr(int ch, int sb, int ss);
        void    put_lr(double data, int ch, int sb, int ss);
        // Read/Write Reordered data
        double  get_re(int ch, int sb, int ss);
        void    put_re(double data, int ch, int sb, int ss);
        // Read/Write Antialiased data
        double  get_hybridIn(int ch, int sb, int ss);
        void    put_hybridIn(double data, int ch, int sb, int ss);
        // Read/Write Hybrid
        double  get_hybridOut(int ch, int sb, int ss);
        void    put_hybridOut(double data, int ch, int sb, int ss);
        // Read/Write Hybrid-Prev
        double  get_prevblck(int ch, int sb, int ss);
        void    put_prevblck(double data, int ch, int sb, int ss);
        // Read/Write Hybrid-Rawout
        double  get_rawout(int ss);
        void    put_rawout(double data, int ss);
        // Read/Write PCM Data
        short   get_pcm_sample(int ch, int sb, int ss);
        void    put_pcm_sample(short data, int ch, int sb, int ss);

    // Constructor ///////////////////////////////////////////////////////////
    SC_CTOR(decoder_mem)
    {
        SC_METHOD(proc_mem_init);
        sensitive << clk << rst;
    }
};

#endif
