///////////////////////////////////////////////////////////////////////////
// Filename : sc_III_Decoder_ctl.cpp
// Project  : Ans_MP3_Player/MP3_Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision :
// Comment  : Top Controller of MP3 Decoder
//
//

#include "sc_III_Decoder_ctl.h"

void decoder_ctl::proc_decoder_ctl()
{
    // Decoder Ctl FSM //////////////////////////////////////
    if (rst.read()==(sc_bit)'0')    // Async. Reset
    {
        nFrame = 0;

        S_III_Decoder   = S_STOP_WAIT;
        hdr_start       = (sc_bit)'0';
        scl_start       = (sc_bit)'0';
        huff_start      = (sc_bit)'0';
        deq_start       = (sc_bit)'0';
        stereo_start    = (sc_bit)'0';
        antialias_start = (sc_bit)'0';
        hybrid_start    = (sc_bit)'0';
        subband_start   = (sc_bit)'0';
        pcmout_start    = (sc_bit)'0';

        ch = false;
        gr = false;

        mem_init = (sc_bit)'0';
        fill_stream = (sc_bit)'0';
    }
    else if (clk.event() && clk.read()==(sc_bit)'1')    // Clocking Rising-Edge
    {
        switch(S_III_Decoder.read())
        {
            case S_STOP_WAIT:
                if (stop.read()==(sc_bit)'0')   // Stop Button
                {
#ifdef _DEBUG_MSG
                    cout << "[" << nFrame << "][" << gr << "][" << ch << "] STOP_WAIT" << endl;
#endif
                    mem_init = (sc_bit)'1';
                    S_III_Decoder = S_INIT_MEM;
                }
                break;

            case S_INIT_MEM:
                if (m_init_done.read()==(sc_bit)'1')
                {
#ifdef _DEBUG_MSG
                    cout << "[" << nFrame << "][" << gr << "][" << ch << "] INIT_MEM" << endl;
#endif
                    mem_init = (sc_bit)'0';
                    S_III_Decoder = S_READ_MP3;
                }
                break;

            case S_READ_MP3:
#ifdef _DEBUG_MSG
                cout << "[" << nFrame << "][" << gr << "][" << ch << "] READ_MP3" << endl;
#endif
                fill_stream = (sc_bit)'1';
                S_III_Decoder = S_WAIT_FILL_STREAM;
                break;

            case S_WAIT_FILL_STREAM:
                if (stream_full.read()==(sc_bit)'1')
                {
#ifdef _DEBUG_MSG
                    cout << "[" << nFrame << "][" << gr << "][" << ch << "] WAIT_FILL_STREAM" << endl;
#endif
                    fill_stream = (sc_bit)'0';
                    ch = false;
                    gr = false;
                    hdr_start = (sc_bit)'1';
                    S_III_Decoder = S_START_HEADER;
                }
                break;

            case S_START_HEADER:
                if (hdr_ready.read()==(sc_bit)'0')
                {
#ifdef _DEBUG_MSG
                    cout << "[" << nFrame << "][" << gr << "][" << ch << "] START_HEADER" << endl;
#endif
                    hdr_start = (sc_bit)'0';
                    S_III_Decoder = S_WAIT_HEADER;
                }
                break;

            case S_WAIT_HEADER:
                if (hdr_ready.read()==(sc_bit)'1')
                {
#ifdef _DEBUG_MSG
                    cout << "[" << nFrame << "][" << gr << "][" << ch << "] WAIT_HEADER" << endl;
#endif
                    scl_start = (sc_bit)'1';
                    S_III_Decoder = S_START_SCALE;
                }
                break;

            case S_START_SCALE:
                if (scl_ready.read()==(sc_bit)'0')
                {
#ifdef _DEBUG_MSG
                    cout << "[" << nFrame << "][" << gr << "][" << ch << "] START_SCALE" << endl;
#endif
                    scl_start = (sc_bit)'0';
                    S_III_Decoder = S_WAIT_SCALE;
                }
                break;

            case S_WAIT_SCALE:
                if (scl_ready.read()==(sc_bit)'1')
                {
#ifdef _DEBUG_MSG
                    cout << "[" << nFrame << "][" << gr << "][" << ch << "] WAIT_SCALE" << endl;
#endif
                    huff_start = (sc_bit)'1';
                    S_III_Decoder = S_START_HUFF;
                }
                break;

            case S_START_HUFF:
                if (huff_ready.read()==(sc_bit)'0')
                {
#ifdef _DEBUG_MSG
                    cout << "[" << nFrame << "][" << gr << "][" << ch << "] START_HUFF" << endl;
#endif
                    huff_start = (sc_bit)'0';
                    S_III_Decoder = S_WAIT_HUFF;
                }
                break;

            case S_WAIT_HUFF:
                if (huff_ready.read()==(sc_bit)'1')
                {
#ifdef _DEBUG_MSG
                    cout << "[" << nFrame << "][" << gr << "][" << ch << "] WAIT_HUFF" << endl;
#endif
                    deq_start = (sc_bit)'1';
                    S_III_Decoder = S_START_DEQ;
                }
                break;

            case S_START_DEQ:
                if (deq_ready.read()==(sc_bit)'0')
                {
#ifdef _DEBUG_MSG
                    cout << "[" << nFrame << "][" << gr << "][" << ch << "] START_DEQ" << endl;
#endif
                    deq_start = (sc_bit)'0';
                    S_III_Decoder = S_WAIT_DEQ;
                }
                break;

            case S_WAIT_DEQ:
                if (deq_ready.read()==(sc_bit)'1')
                {
#ifdef _DEBUG_MSG
                    cout << "[" << nFrame << "][" << gr << "][" << ch << "] WAIT_DEQ" << endl;
#endif
                    S_III_Decoder = S_CHECK_ST;
                }
                break;

            case S_CHECK_ST:
#ifdef _DEBUG_MSG
                cout << "[" << nFrame << "][" << gr << "][" << ch << "] CHECK_ST" << endl;
#endif
                if (stereo.read()==(sc_bit)'1' && ch==false)    // STEREO
                {
                    ch = true;
                    scl_start = (sc_bit)'1';
                    S_III_Decoder = S_START_SCALE;
                }
                else
                {
                    stereo_start = (sc_bit)'1';
                    S_III_Decoder = S_START_STEREO;
                }
                break;

            case S_START_STEREO:
                if (stereo_ready.read()==(sc_bit)'0')
                {
#ifdef _DEBUG_MSG
                    cout << "[" << nFrame << "][" << gr << "][" << ch << "] START_STEREO" << endl;
#endif
                    ch = false;
                    fill_stream = (sc_bit)'1';
                    stereo_start = (sc_bit)'0';
                    S_III_Decoder = S_WAIT_STEREO;
                }
                break;

            case S_WAIT_STEREO:
                if (stereo_ready.read()==(sc_bit)'1')
                {
#ifdef _DEBUG_MSG
                    cout << "[" << nFrame << "][" << gr << "][" << ch << "] WAIT_STEREO" << endl;
#endif
                    reorder_start = (sc_bit)'1';
                    S_III_Decoder = S_START_REORDER;
                }
                break;

            case S_START_REORDER:
                if (reorder_ready.read()==(sc_bit)'0')
                {
#ifdef _DEBUG_MSG
                    cout << "[" << nFrame << "][" << gr << "][" << ch << "] START_REORDER" << endl;
#endif
                    reorder_start = (sc_bit)'0';
                    S_III_Decoder = S_WAIT_REORDER;
                }
                break;

            case S_WAIT_REORDER:
                if (reorder_ready.read()==(sc_bit)'1')
                {
#ifdef _DEBUG_MSG
                    cout << "[" << nFrame << "][" << gr << "][" << ch << "] WAIT_REORDER" << endl;
#endif
                    antialias_start = (sc_bit)'1';
                    S_III_Decoder = S_START_ANTIALIAS;
                }
                break;

            case S_START_ANTIALIAS:
                if (antialias_ready.read()==(sc_bit)'0')
                {
#ifdef _DEBUG_MSG
                    cout << "[" << nFrame << "][" << gr << "][" << ch << "] START_ANTIALIAS" << endl;
#endif
                    antialias_start = (sc_bit)'0';
                    S_III_Decoder = S_WAIT_ANTIALIAS;
                }
                break;

            case S_WAIT_ANTIALIAS:
                if (antialias_ready.read()==(sc_bit)'1')
                {
#ifdef _DEBUG_MSG
                    cout << "[" << nFrame << "][" << gr << "][" << ch << "] WAIT_ANTIALIAS" << endl;
#endif
                    hybrid_start = (sc_bit)'1';
                    S_III_Decoder = S_START_HYBRID;
                }
                break;

            case S_START_HYBRID:
                if (hybrid_ready.read()==(sc_bit)'0')
                {
#ifdef _DEBUG_MSG
                    cout << "[" << nFrame << "][" << gr << "][" << ch << "] START_HYBRID" << endl;
#endif
                    hybrid_start = (sc_bit)'0';
                    S_III_Decoder = S_WAIT_HYBRID;
                }
                break;

            case S_WAIT_HYBRID:
                if (hybrid_ready.read()==(sc_bit)'1')
                {
#ifdef _DEBUG_MSG
                    cout << "[" << nFrame << "][" << gr << "][" << ch << "] WAIT_HYBRID" << endl;
#endif
                    if (stereo.read()==(sc_bit)'1' && ch==false)    // STEREO
                    {
                        ch = true;
                        reorder_start = (sc_bit)'1';
                        S_III_Decoder = S_START_REORDER;
                    }
                    else
                    {
                        subband_start = (sc_bit)'1';
                        S_III_Decoder = S_START_SUBBAND;
                    }
                }
                break;

            case S_START_SUBBAND:
                if (subband_ready.read()==(sc_bit)'0')
                {
#ifdef _DEBUG_MSG
                    cout << "[" << nFrame << "][" << gr << "][" << ch << "] START_SUBBAND" << endl;
#endif
                    subband_start = (sc_bit)'0';
                    S_III_Decoder = S_WAIT_SUBBAND;
                }
                break;

            case S_WAIT_SUBBAND:
                if (subband_ready.read()==(sc_bit)'1')
                {
#ifdef _DEBUG_MSG
                    cout << "[" << nFrame << "][" << gr << "][" << ch << "] WAIT_SUBBAND" << endl;
#endif
                    pcmout_start = (sc_bit)'1';
                    S_III_Decoder = S_START_PCMOUT;
                }
                break;

            case S_START_PCMOUT:
#ifdef _DEBUG_MSG
                cout << "[" << nFrame << "][" << gr << "][" << ch << "] PCM_OUT" << endl;
#endif
                pcmout_start = (sc_bit)'0';
                S_III_Decoder = S_CHECK_VERSION;
                break;

            case S_CHECK_VERSION:
#ifdef _DEBUG_MSG
                cout << "[" << nFrame << "][" << gr << "][" << ch << "] CHECK_VERSION" << endl;
#endif
                ch = false;
                if (version.read()==(sc_bit)'1' && gr==false) // MPEG Version 1
                {
                    gr = true;

                    scl_start = (sc_bit)'1';
                    S_III_Decoder = S_START_SCALE;
                }
                else     // MPEG Version 2
                {
                    gr = false;
                    hdr_start = (sc_bit)'1';
                    S_III_Decoder = S_WAIT_FILL_STREAM1;
                }
                break;

            case S_WAIT_FILL_STREAM1:
                if (stream_full.read()==(sc_bit)'1')
                {
#ifdef _DEBUG_MSG
                    cout << "[" << nFrame << "][" << gr << "][" << ch << "] WAIT_FILL_STREAM1" << endl;
#endif
                    S_III_Decoder = S_WAIT_FILL_STREAM2;
                }
                break;

            case S_WAIT_FILL_STREAM2:
#ifdef _DEBUG_MSG
                cout << "[" << nFrame << "][" << gr << "][" << ch << "] WAIT_FILL_STREAM2" << endl;
#endif
                fill_stream = (sc_bit)'0';
                S_III_Decoder = S_STOP_PAUSE;
                break;

            case S_STOP_PAUSE:
                if (stop.read()==(sc_bit)'1')   // Stop Button
                {
#ifdef _DEBUG_MSG
                    cout << "[" << nFrame << "][" << gr << "][" << ch << "] Decoder Stopped!!!" << endl;
#endif
                    Done();
                    sc_stop();
                    S_III_Decoder = S_STOP_WAIT;
                }
                else
                {
#ifdef _DEBUG_MSG
                    cout << "[" << nFrame++ << "][" << gr << "][" << ch << "] STOP_PAUSE" << endl;
#endif
                    gr = false;
                    ch = false;
                    hdr_start = (sc_bit)'1';
                    S_III_Decoder = S_START_HEADER;
                }
                break;

            default:
                S_III_Decoder = S_WAIT_HEADER;
                break;
        }
    }

    channel = ch;
    granule = gr;
}
