///////////////////////////////////////////////////////////////////////////
// Filename : sc_III_Dequantizer.cpp
// Project  : MP3 Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision :
// Comment  :
//
//

#include "math.h"
#include "MP3_Decoder.h"
#include "sc_III_Dequantizer.h"
#include "sfBandIndex.h"

#include "sc_III_Decoder_Mem.h"

void iii_dequantizer::proc_deq()
{
    int t_sampling_frequency    = (int)(sc_uint<2>)sampling_frequency.read();
    int t_version               = version.read();
    int t_window_switching_flag = window_switching_flag.read();
    int t_block_type            = (int)(sc_uint<2>)block_type.read();
    int t_mixed_block_flag      = mixed_block_flag.read();
    int t_global_gain           = (int)(sc_uint<8>)global_gain.read();
    int t_scalefac_scale        = scalefac_scale.read();
    int t_preflag               = preflag.read();
    int t_channel               = channel.read();
    int t_subblock_gain[3];

    if (rst.read()==(sc_bit)'0')    // Async. Reset
    {
        Deq_State = S_DEQ_WAIT;
        ready = (sc_bit)'1';
    }
    else if (clk.event() && clk.read()==(sc_bit)'1')    // Clocking Rising-Edge
    {
        switch(Deq_State.read())
        {
            case S_DEQ_WAIT:
                if (start.read()==(sc_bit)'1')
                {
                    ready = (sc_bit)'0';
                    Deq_State = S_DEQ_DECODE;
                }
                break;

            case S_DEQ_DECODE:
                if (start.read()==(sc_bit)'0')
                {
                    t_subblock_gain[0] = (int)(sc_uint<3>)subblock_gain0.read();
                    t_subblock_gain[1] = (int)(sc_uint<3>)subblock_gain1.read();
                    t_subblock_gain[2] = (int)(sc_uint<3>)subblock_gain2.read();

                    III_dequantize_sample(
                        t_sampling_frequency,
                        t_version,
                        t_window_switching_flag,
                        t_block_type,
                        t_mixed_block_flag,
                        t_global_gain,
                        t_subblock_gain,
                        t_scalefac_scale,
                        t_preflag,
                        t_channel);

                    Deq_State = S_DEQ_DOWNLOAD;
                }
                break;

            case S_DEQ_DOWNLOAD:
                Deq_State = S_DEQ_BUSY;
#ifdef _DEBUG_OUT
                {
                    FILE *fp;
                    int ss, sb;

                    fp = fopen("dbg_requantize.txt", "a+");

                    fprintf(fp, "Frame=%d/Granule=%d/Channel=%d\n",
                                    u_mp3_decoder.u_decoder_ctl->nFrame,
                                    ((u_mp3_decoder.u_decoder_ctl->gr)? 1:0),
                                    ((u_mp3_decoder.u_decoder_ctl->ch)? 1:0) );

                    for (sb=0; sb<SBLIMIT; sb++)
                        for (ss=0; ss<SSLIMIT; ss++)
                            fprintf(fp, "[%d][%2d][%2d] %16.12f\n", t_channel, sb, ss,
                                        u_mp3_decoder.u_decoder_mem->get_ro(t_channel, sb, ss));
                    fflush(fp);
                    fclose(fp);
                }
#endif
                break;

            case S_DEQ_BUSY:
                ready = (sc_bit)'1';
                Deq_State = S_DEQ_WAIT;
                break;

            default:
                Deq_State = S_DEQ_WAIT;
                break;
        }
    }
}

//////////////////////////////////////////////////////////////////////////
// Dequantizer for Layer III
//
int pretab[22] = {0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,2,2,3,3,3,2,0};

void iii_dequantizer::III_dequantize_sample(
    int sampling_frequency,
    int version,
    int window_switching_flag,
    int block_type,
    int mixed_block_flag,
    int global_gain,
    int subblock_gain[3],
    int scalefac_scale,
    int preflag,
    int ch)
{
    int ss,sb,cb=0,sfreq;
    int next_cb_boundary, cb_begin, cb_width, sign;

    double  xr;

    sfreq = sampling_frequency + (version * 3);

    // choose correct scalefactor band per block type, initalize boundary 
    if (window_switching_flag && (block_type == 2) )
        if (mixed_block_flag) 
            next_cb_boundary=sfBandIndex[sfreq].l[1];  // LONG blocks
        else
        {
            next_cb_boundary = sfBandIndex[sfreq].s[1]*3; // pure SHORT block
            cb_width = sfBandIndex[sfreq].s[1];
            cb_begin = 0;
        }
    else 
        next_cb_boundary = sfBandIndex[sfreq].l[1];  // LONG blocks

    // apply formula per block type
    for (sb=0 ; sb < SBLIMIT ; sb++)
    {
        for (ss=0 ; ss < SSLIMIT ; ss++)
        {
            if ( (sb*18)+ss == next_cb_boundary)    // Adjust critical band boundary
            {
                if (window_switching_flag && (block_type == 2))
                {
                    if (mixed_block_flag)
                    {
                        if (((sb*18)+ss) == sfBandIndex[sfreq].l[8])
                        {
                            next_cb_boundary = sfBandIndex[sfreq].s[4]*3; 
                            cb = 3;
                            cb_width = sfBandIndex[sfreq].s[cb+1] - sfBandIndex[sfreq].s[cb];
                            cb_begin = sfBandIndex[sfreq].s[cb]*3;
                        }
                        else if (((sb*18)+ss) < sfBandIndex[sfreq].l[8]) 
                            next_cb_boundary = sfBandIndex[sfreq].l[(++cb)+1];
                        else
                        {
                            next_cb_boundary = sfBandIndex[sfreq].s[(++cb)+1]*3;
                            cb_width = sfBandIndex[sfreq].s[cb+1] - sfBandIndex[sfreq].s[cb];
                            cb_begin = sfBandIndex[sfreq].s[cb]*3;      
                        }   
                    }
                    else
                    {
                        next_cb_boundary = sfBandIndex[sfreq].s[(++cb)+1]*3;
                        cb_width = sfBandIndex[sfreq].s[cb+1] - sfBandIndex[sfreq].s[cb];
                        cb_begin = sfBandIndex[sfreq].s[cb]*3;
                    } 
                }
                else // Long blocks
                    next_cb_boundary = sfBandIndex[sfreq].l[(++cb)+1];
            }

            // Compute overall (global) scaling.
            xr = pow( 2.0 , (0.25 * (global_gain - 210.0)));

            // Do long/short dependent scaling operations.
            if (window_switching_flag && (
                    ((block_type == 2) && (mixed_block_flag == 0)) ||
                    ((block_type == 2) && mixed_block_flag && (sb >= 2)) ))
            {
                xr *= pow(2.0, 0.25 * -8.0 * subblock_gain[(((sb*18)+ss) - cb_begin)/cb_width]);
                xr *= pow(2.0, 0.25 * -2.0 * (1.0+scalefac_scale)
                                    * u_mp3_decoder.u_decoder_mem->get_scalefac(
                                            ch, (((sb*18)+ss) - cb_begin)/cb_width, cb, 1) );
            }
            else    // LONG block types & 1st 2 subbands of switched blocks
            {   
                xr *= pow(2.0, -0.5 * (1.0+scalefac_scale)
                                    * ( u_mp3_decoder.u_decoder_mem->get_scalefac(ch, 0, cb, 0)
                                        + preflag*pretab[cb]) );
            }

            // Scale quantized value.
            sign = (u_mp3_decoder.u_decoder_mem->get_is( ch, sb, ss)<0) ? 1 : 0;
            xr *= pow( (double) abs(u_mp3_decoder.u_decoder_mem->get_is( ch, sb, ss)), ((double)4.0/3.0) );
            if (sign)
                xr = -xr;

            // Finally Write dequantized value
            u_mp3_decoder.u_decoder_mem->put_ro( xr, ch, sb, ss);
        }
    }
}
