///////////////////////////////////////////////////////////////////////////
// Filename : sc_III_Frame_buffer.cpp
// Project  : MP3 Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision :
// Comment  :
//
//

#include "MP3_Decoder.h"
#include "sc_III_Frame_buffer.h"

// Fill Frame Memory /////////////////////////////////////////////////////
void frame_buffer::proc_fill_buffer()
{
    static int  t_End_FIFO = 0;

    if (rst.read()==(sc_bit)'0')    // Async. Reset
    {
        // Frame Memory (1024 Bytes)
        nBegin_FIFO   = 0;
        nEnd_FIFO     = 0;
        frm_bit_index = 7;
        frm_totbit    = 0;

        // Main Data Buffer (4096 Bytes)
        buf_offset     = 0;
        buf_byte_index = 0;
        buf_bit_index  = 7;
        buf_totbit     = 0;

        t_End_FIFO = 0;
        stream_full = (sc_bit)'0';
        mp3_req = (sc_bit)'0';
        Fill_Buff_State = S_FILL_BUFF_WAIT;
    }
    else if (clk.event() && clk.read()==(sc_bit)'1')    // Clocking Rising-Edge
    {
        switch(Fill_Buff_State.read())
        {
            case S_FILL_BUFF_WAIT:
                if (fill_stream.read()==(sc_bit)'1')
                {
                    t_End_FIFO = (nEnd_FIFO+1) % 1024;

                    if (nEnd_FIFO==nBegin_FIFO || t_End_FIFO!=nBegin_FIFO)
                    {
                        stream_full = (sc_bit)'0';
                        mp3_req = (sc_bit)'1';
                        Fill_Buff_State = S_FILL_BUFF_REQ;
                    }
                    else
                    {
                        //printf("\nFIFO: Begin=%d End=%d", nBegin_FIFO, nEnd_FIFO);
                        stream_full = (sc_bit)'1';
                    }
                }
                break;

            case S_FILL_BUFF_REQ:
                if (mp3_ack.read()==(sc_bit)'1')
                {
                    frm_mem[nEnd_FIFO] = mp3_data;
                    nEnd_FIFO = t_End_FIFO;
                    mp3_req = (sc_bit)'0';
                    Fill_Buff_State = S_FILL_BUFF_ACK;
                }
                break;

            case S_FILL_BUFF_ACK:
                if (mp3_ack.read()==(sc_bit)'0')
                    Fill_Buff_State = S_FILL_BUFF_WAIT;
                break;
        }
    }
}

// Extract 1-Bits from Frame Memory //////////////////////
unsigned long frame_buffer::get1bit()
{
    unsigned int    c;
    unsigned long   bit_return;

    //cout << "get1bit[" << nBegin_FIFO << "][" << nEnd_FIFO << "]" << endl;

    c = (unsigned int)(sc_uint<8>)frm_mem[nBegin_FIFO];
    bit_return = (c >> frm_bit_index) & 0x0001;

    frm_bit_index--;
    if ((int)frm_bit_index<0)
    {
        frm_bit_index = 7;
        nBegin_FIFO++;
    }

    if (nBegin_FIFO==1024)
        nBegin_FIFO = 0;

    frm_totbit++;

    return(bit_return);
}

// Extract N-Bits from Frame Memory //////////////////////
unsigned long frame_buffer::getbits(int N)
{
    int             i;
    unsigned long   bits_return = 0;

    for (i=0; i<N; i++)
    {
        bits_return <<= 1;
        bits_return |= get1bit();
    }
    return(bits_return);
}

// Return the current bit stream length (in bits) of Frame Memory
unsigned long frame_buffer::sstell()
{
    return(frm_totbit);
}

// End of Frame Memory ///////////////////////////////////
unsigned long frame_buffer::end_bs()
{
    return((nEnd_FIFO==nBegin_FIFO)? 1 : 0);
}

// Write N bits into Main Data Buffer /////////////////////////////
void frame_buffer::hputbuf(unsigned int val)
{
    buf_mem[buf_offset % 4096] = val;
    buf_offset++;
}

// Return the current bit stream length (in bits) of Main Data Buffer
unsigned long frame_buffer::hsstell()
{
    return(buf_totbit);
}

// Extract 1-Bits from Main Data Buffer /////////////////////////////
unsigned long frame_buffer::hget1bit()
{
    unsigned int    c;
    unsigned long   bit_return;

    //cout << "hget1bit[" << buf_byte_index << "][" << buf_bit_index << "]" << endl;

    c = (unsigned int)(sc_uint<8>)buf_mem[buf_byte_index];
    bit_return = (c >> buf_bit_index) & 0x0001;

    buf_bit_index--;
    if ((int)buf_bit_index<0)
    {
        buf_bit_index = 7;
        buf_byte_index++;
    }

    if (buf_byte_index==4096)
        buf_byte_index = 0;

    buf_totbit++;

    return(bit_return);
}

// Extract N-Bits from Main Data Buffer ////////////////////////////////
unsigned long frame_buffer::hgetbits(int N)
{
    int             i;
    unsigned long   bits_return = 0;

    for (i=0; i<N; i++)
    {
        bits_return <<= 1;
        bits_return |= hget1bit();
    }
    return(bits_return);
}

// Align/Rewind Main Data Buffer ///////////////////////////////////////
void frame_buffer::rewindNbytes(int N)
{
    buf_totbit     -= N*8;
    buf_byte_index -= N;
}

void frame_buffer::rewindNbits(int N)
{
    buf_totbit    -= N;
    buf_bit_index += N;
    
    while( buf_bit_index >= 8 )
    {
        buf_bit_index -= 8;
        buf_byte_index--;
    }
}
