///////////////////////////////////////////////////////////////////////////
// Filename : sc_III_HeaderInfo.cpp
// Project  : MP3 Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision :
// Comment  :
//
//

#ifndef SOFT
    extern "C" {
        int __lprintf();
        int __dprintf();
    }
#endif

#include "MP3_Decoder.h"
#include "sc_III_HeaderInfo.h"
#include "sc_III_Frame_buffer.h"

void iii_headerinfo::proc_header()
{

    if (rst.read()==(sc_bit)'0')    // Async. Reset
    {
        frame_start = 0;

        Header_State = S_HEADER_WAIT;
        ready = (sc_bit)'1';
    }
    else if (clk.event() && clk.read()==(sc_bit)'1')    // Clocking Rising-Edge
    {
        switch(Header_State.read())
        {
            case S_HEADER_WAIT:
                if (start.read()==(sc_bit)'1')
                {
                    ready = (sc_bit)'0';
                    Header_State = S_HEADER_DECODE;
                }
                break;

            case S_HEADER_DECODE:
                if (start.read()==(sc_bit)'0')
                {
                    // Find Frame's SYNC from Input Bitstream
                    if (!seek_sync())
                    {
                        cout << ">>>>> SYNC Error : Frame cannot be located <<<<<" << endl;
                    }
                    else
                    {
                        // Get Frame's Header Information
                        decode_info(&fr_ps);
                        hdr_to_frps(&fr_ps);

                        // CRC Error Protection
                        if (info.error_protection)
                            buffer_CRC(&old_crc);

                        // Get Layer III Side Information
                        III_get_side_info(&III_side_info, &fr_ps);

                        // Manage Input Stream Buffer
                        nSlots = main_data_slots(fr_ps);    // Get Slot count in main_data
                        for (; nSlots > 0; nSlots--)        // Read main data.
                            u_mp3_decoder.u_frame_buffer->hputbuf(
                                (unsigned int)u_mp3_decoder.u_frame_buffer->getbits(8));

                        // Set End Position of privious frame
                        main_data_end = u_mp3_decoder.u_frame_buffer->hsstell() / 8;

                        if (flush_main=(u_mp3_decoder.u_frame_buffer->hsstell() % 8))   // Layer III : BitperSlot=8
                        { 
                            u_mp3_decoder.u_frame_buffer->hgetbits((int)(8 - flush_main));
                            main_data_end++;
                        }

                        bytes_to_discard = frame_start - main_data_end - III_side_info.main_data_begin ;
                        //if (main_data_end > 4096)
                        //{
                        //  frame_start -= 4096;
                        //  u_mp3_decoder.u_frame_buffer->rewindNbytes(4096);
                        //}

                        frame_start += main_data_slots(fr_ps);
                        for (; bytes_to_discard > 0; bytes_to_discard--)
                            u_mp3_decoder.u_frame_buffer->hgetbits(8);
                    }

                    // Header Output ///////////////////////
                    proc_header_out();

                    Header_State = S_HEADER_DOWNLOAD;
                }
                break;

            case S_HEADER_DOWNLOAD:
                Header_State = S_HEADER_BUSY;
                break;

            case S_HEADER_BUSY:
                ready = (sc_bit)'1';
                Header_State = S_HEADER_WAIT;
                break;

            default:
                Header_State = S_HEADER_WAIT;
                break;
        }
    }
}


void iii_headerinfo::proc_header_out()
{
    int t_ch = ch.read();
    int t_gr = gr.read();

    // Header Info : Stereo, Mode, Mode Ext., Version, Sampling Freq.
    stereo             = (bool)(fr_ps.stereo - 1);
    bitrate_index      = (int)fr_ps.header->bitrate_index;
    mode               = (int)fr_ps.header->mode;
    mode_ext           = (int)fr_ps.header->mode_ext;
    version            = (bool)fr_ps.header->version;
    sampling_frequency = (int)fr_ps.header->sampling_frequency;

    // Side Informations
    big_values            = (int)III_side_info.ch[t_ch].gr[t_gr].big_values;
    block_type            = (int)III_side_info.ch[t_ch].gr[t_gr].block_type;
    count1table_select    = (bool)III_side_info.ch[t_ch].gr[t_gr].count1table_select;
    global_gain           = (int)III_side_info.ch[t_ch].gr[t_gr].global_gain;
    subblock_gain0        = (int)III_side_info.ch[t_ch].gr[t_gr].subblock_gain[0];
    subblock_gain1        = (int)III_side_info.ch[t_ch].gr[t_gr].subblock_gain[1];
    subblock_gain2        = (int)III_side_info.ch[t_ch].gr[t_gr].subblock_gain[2];
    mixed_block_flag      = (bool)III_side_info.ch[t_ch].gr[t_gr].mixed_block_flag;
    part2_3_length        = (int)III_side_info.ch[t_ch].gr[t_gr].part2_3_length;
    preflag               = (bool)III_side_info.ch[t_ch].gr[t_gr].preflag;
    region0_count         = (int)III_side_info.ch[t_ch].gr[t_gr].region0_count;
    region1_count         = (int)III_side_info.ch[t_ch].gr[t_gr].region1_count;
    scalefac_compress     = (int)III_side_info.ch[t_ch].gr[t_gr].scalefac_compress;
    scalefac_scale        = (bool)III_side_info.ch[t_ch].gr[t_gr].scalefac_scale;
    scfsi0                = (bool)III_side_info.ch[t_ch].scfsi[0];
    scfsi1                = (bool)III_side_info.ch[t_ch].scfsi[1];
    scfsi2                = (bool)III_side_info.ch[t_ch].scfsi[2];
    scfsi3                = (bool)III_side_info.ch[t_ch].scfsi[3];
    table_select0         = (int)III_side_info.ch[t_ch].gr[t_gr].table_select[0];
    table_select1         = (int)III_side_info.ch[t_ch].gr[t_gr].table_select[1];
    table_select2         = (int)III_side_info.ch[t_ch].gr[t_gr].table_select[2];
    window_switching_flag = (bool)III_side_info.ch[t_ch].gr[t_gr].window_switching_flag;
}

// Seek SYNC Bits //////////////////////////////////////////////////
unsigned long iii_headerinfo::seek_sync()
{
    static int frameCnt=0;
    unsigned long   aligning;
    unsigned long   val;
    long            maxi = 0x0FFF;  /* SYNC : 4095 = "1111_1111_1111" */

#ifndef _DEBUG_MSG
    // Display Decoding Process -----------------------------
    static int  HeaderNum = 0;

    switch(HeaderNum%4)
    {
        case 0:
            fprintf(stderr, "\015-");
            break;
        case 1:
            fprintf(stderr, "\015\\");
            break;
        case 2:
            fprintf(stderr, "\015|");
            break;
        case 3:
            fprintf(stderr, "\015/");
            break;
    }
    HeaderNum++;
    fflush(stderr); 
#endif

#ifdef SOFT
    aligning = u_mp3_decoder.u_frame_buffer->sstell()%8;    // BYTE Aligning
    if (aligning)
        u_mp3_decoder.u_frame_buffer->getbits((int)(8-aligning));
#endif

    val = u_mp3_decoder.u_frame_buffer->getbits(12);    // Read SYNC Bits

    while (((val&maxi) != maxi) && (!u_mp3_decoder.u_frame_buffer->end_bs()))
    {
#ifdef SOFT
        fprintf(stderr, "Seek Sync again...\n");
        val <<= 8;
        val |= u_mp3_decoder.u_frame_buffer->getbits(8);
#else
        val <<= 1;
        val |= u_mp3_decoder.u_frame_buffer->get1bit();
#endif
    }

#ifdef SOFT
    if(frameCnt % 100 == 0) printf("{%d}\n", frameCnt);
    frameCnt++;
#else
    if(frameCnt % 100 == 0) __dprintf("{%d}\n",frameCnt);
    frameCnt++;
#endif

    if (u_mp3_decoder.u_frame_buffer->end_bs())
        return(0);
    else
        return(1);
}

/////////////////////////////////////////////////////////////////////////////////
//  Decode Header Information
//
//  this function read 20-Bits Header Information after SYNC and
//  places the bit stream pointer right after the header info.
//
//   | Frame Start (Byte Aligned)
//   |<-- SYNC ---->|
//   +--------------+-----------------------------------------+-----
//   |1111_1111_1111| Header Info                             |
//   +--------------+-----------------------------------------+-----
//   |<----------------- 32-Bits Header --------------------->|
//
//  Header (32-Bits)
//      +- SyncWord (12)                                                         
//      +- ID (1)                                                                
//      +- Layer (2) - Layer I("11")/II("10")/III("01")                          
//      +- Protection_Bit (1) - CRC Protection ('1' - NO, '0' - YES)             
//      +- Bitrste_Index (4) - Bitrate Index ("1100" - 128k)                     
//      +- Sampling_Frequency (2) - "00"(22.05),"01"(24),"10"(16), "11"(Reserved)
//      |                           "00"(44.1), "01"(48),"10"(32), "11"(Reserved)
//      +- Padding_Bit (1)                                                       
//      +- Private_Bit (1)                                                       
//      +- Mode (2) - Stereo("00"),J-Stereo("01"),Dual Ch(10),Mono(11) Mode      
//      +- Mode_Extension (2) - "00"-Non-Stereo,"01"-Intensity Stereo,           
//      |                       "10"-MS-Stereo, "11"-both                        
//      |       when mode=J-Stereo, Intensity Stereo and MS-Stereo "off" both.   
//      +- Copyright (1)                                                         
//      +- original/copy (1)                                                     
//      +- Emphais (2)                                                           
//
void iii_headerinfo::decode_info(frame_params *fr_ps)
{
    layer *hdr = fr_ps->header;
    
    hdr->version            = u_mp3_decoder.u_frame_buffer->get1bit();
    hdr->lay                = 4 - u_mp3_decoder.u_frame_buffer->getbits(2);
    hdr->error_protection   = !u_mp3_decoder.u_frame_buffer->get1bit();
    hdr->bitrate_index      = u_mp3_decoder.u_frame_buffer->getbits(4);
    hdr->sampling_frequency = u_mp3_decoder.u_frame_buffer->getbits(2);
    hdr->padding            = u_mp3_decoder.u_frame_buffer->get1bit();
    hdr->extension          = u_mp3_decoder.u_frame_buffer->get1bit();
    hdr->mode               = u_mp3_decoder.u_frame_buffer->getbits(2);
    hdr->mode_ext           = u_mp3_decoder.u_frame_buffer->getbits(2);
    hdr->copyright          = u_mp3_decoder.u_frame_buffer->get1bit();
    hdr->original           = u_mp3_decoder.u_frame_buffer->get1bit();
    hdr->emphasis           = u_mp3_decoder.u_frame_buffer->getbits(2);

    // Print Header Information /////////////////////////////////////////////////////
    static bool print_header_info = false;
    static int bitrate[2][16] = {
            { 0,  8, 16, 24, 32, 40, 48, 56,  64,  80,  96, 112, 128, 144, 160, 0 },    // MPEG-2
            { 0, 32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 0 } };  // MPEG-1
    static double freq_sampling[2][4] = {
            { 22.05, 24, 16, 0 },   // MPEG-2
            { 44.1,  48, 32, 0 } }; // MPEG-1
    static char *str_mode_ext[4] = { "", "[Intensity]", "[MS]", "[Intensity-MS]" };
    static char *str_stereo[4]   = { "Stereo", "J-Stereo", "Dual Ch", "Single Ch" };

    char fs_buff[16];

    if (!print_header_info)
    {
        // Header Info ////////////////////////////
        cout << "MPEG-" << ((hdr->version)? 1 : 2) << endl;
        cout << "Bitrate : " << bitrate[hdr->version][hdr->bitrate_index] << "Kbps" << endl;
        cout << "Sampling Frequency : " << freq_sampling[hdr->version][hdr->sampling_frequency] << "K" << endl;
        cout << "Mode(Stereo) : " << str_stereo[hdr->mode] << str_mode_ext[hdr->mode_ext] << endl;

        // PCM data output filename ///////////////
        if (hdr->mode<3)
            strcat(u_mp3_decoder.pcm_filename, ".stereo.");
        else
            strcat(u_mp3_decoder.pcm_filename, ".mono.");
        sprintf(fs_buff, "%d", (int)freq_sampling[hdr->version][hdr->sampling_frequency]);
        strcat(u_mp3_decoder.pcm_filename, fs_buff);
        strcat(u_mp3_decoder.pcm_filename, ".pcm");

        print_header_info = true;
    }
}

///////////////////////////////////////////////////////////////////////////////////
//  Header Info. to Frame Parameter
//  interpret data in hdr str to fields in fr_ps
//
void iii_headerinfo::hdr_to_frps(frame_params *fr_ps)
{
    static int jsb_table[4] = { 0, 4, 8, 16};   // lay+m_e -> jsbound

    layer *hdr = fr_ps->header;

    fr_ps->actual_mode = hdr->mode;
    fr_ps->stereo = (hdr->mode == MPG_MD_MONO) ? 1 : 2;
    fr_ps->sblimit = SBLIMIT;           /* Subband Limits set to 32 */

    if(hdr->mode == MPG_MD_JOINT_STEREO)
        fr_ps->jsbound = jsb_table[hdr->mode_ext];  /* J-Stereo Subband Bound */
    else
        fr_ps->jsbound = fr_ps->sblimit;
}

/////////////////////////////////////////////////////////////////////////////////
// CRC Error Protection
//
void iii_headerinfo::buffer_CRC(unsigned int  *old_crc)
{
    *old_crc = u_mp3_decoder.u_frame_buffer->getbits(16);
}

/////////////////////////////////////////////////////////////////////////////////
//  Get Side Information of Layer III
//  ---------------------------------
//  Read following ......
//
//  - main_data_begin
//  - private_bits
//  - scfsi[ch][scfsi_band]
//  - part2_3_length[gr][ch]
//  - big_value[gr][ch]
//  - global_gain[gr][ch]
//  - scalefac_compress[gr][ch]
//  - window_switching_flag[gr][ch]
//
//  if (window_switching_flag)
//      - block_type[gr][ch]
//      - mixed_block_flag[gr][ch]
//      - table_select[gr][ch][region]
//      - subblock_gain[gr][ch][window]
//        n_region=2, n_window=3
//  else
//      - table_select[gr][ch][region]
//      - region0_count[gr][ch]
//      - region1_count[gr][ch]
//        n_region=3
//
//  - preflag[gr][ch]
//  - scalefac_scale[gr][ch]
//  - count1table_select[gr][ch]
//
//  n_ch=2 for Stereo, n_ch=1 for Mono
//  n_gr=1 for MPEG-2, n_gr=2 for MPEG-1
//
void iii_headerinfo::III_get_side_info(III_side_info_t *si, frame_params *fr_ps )
{
    int ch, gr, i;
    int stereo = fr_ps->stereo;
   
    if(fr_ps->header->version != MPEG_PHASE2_LSF)
    { /* MPEG-1 */
        si->main_data_begin = u_mp3_decoder.u_frame_buffer->getbits(9);
        if (stereo == 1)
            si->private_bits = u_mp3_decoder.u_frame_buffer->getbits(5);
        else
            si->private_bits = u_mp3_decoder.u_frame_buffer->getbits(3);

        for (ch=0; ch<stereo; ch++)
            for (i=0; i<4; i++)
                si->ch[ch].scfsi[i] = u_mp3_decoder.u_frame_buffer->get1bit();
 
        for (gr=0; gr< 2 ; gr++)
        {
            for (ch=0; ch<stereo; ch++)
            {
                si->ch[ch].gr[gr].part2_3_length        = u_mp3_decoder.u_frame_buffer->getbits(12);
                si->ch[ch].gr[gr].big_values            = u_mp3_decoder.u_frame_buffer->getbits(9);
                si->ch[ch].gr[gr].global_gain           = u_mp3_decoder.u_frame_buffer->getbits(8);
                si->ch[ch].gr[gr].scalefac_compress     = u_mp3_decoder.u_frame_buffer->getbits(4);
                si->ch[ch].gr[gr].window_switching_flag = u_mp3_decoder.u_frame_buffer->get1bit();

                if (si->ch[ch].gr[gr].window_switching_flag)
                {
                    si->ch[ch].gr[gr].block_type       = u_mp3_decoder.u_frame_buffer->getbits(2);
                    si->ch[ch].gr[gr].mixed_block_flag = u_mp3_decoder.u_frame_buffer->get1bit();
                    for (i=0; i<2; i++)
                        si->ch[ch].gr[gr].table_select[i] = u_mp3_decoder.u_frame_buffer->getbits(5);
                    for (i=0; i<3; i++)
                        si->ch[ch].gr[gr].subblock_gain[i] = u_mp3_decoder.u_frame_buffer->getbits(3);

                    /* Set region_count parameters since they are implicit in this case. */
                    if (si->ch[ch].gr[gr].block_type==0)
                    {
#ifdef SOFT
                        printf("Side info bad: block_type == 0 in split block.\n");
#else
                        __lprintf("Side info bad\n");
#endif
                        exit(0);
                    }
                    else if (si->ch[ch].gr[gr].block_type==2 && si->ch[ch].gr[gr].mixed_block_flag==0)
                        si->ch[ch].gr[gr].region0_count = 8; /* MI 9; */
                    else
                        si->ch[ch].gr[gr].region0_count = 7; /* MI 8; */
                    si->ch[ch].gr[gr].region1_count = 20 - si->ch[ch].gr[gr].region0_count;
                }
                else
                {
                    for (i=0; i<3; i++)
                        si->ch[ch].gr[gr].table_select[i] = u_mp3_decoder.u_frame_buffer->getbits(5);
                    si->ch[ch].gr[gr].region0_count = u_mp3_decoder.u_frame_buffer->getbits(4);
                    si->ch[ch].gr[gr].region1_count = u_mp3_decoder.u_frame_buffer->getbits(3);
                    si->ch[ch].gr[gr].block_type    = 0;
                }

                si->ch[ch].gr[gr].preflag            = u_mp3_decoder.u_frame_buffer->get1bit();
                si->ch[ch].gr[gr].scalefac_scale     = u_mp3_decoder.u_frame_buffer->get1bit();
                si->ch[ch].gr[gr].count1table_select = u_mp3_decoder.u_frame_buffer->get1bit();
            }
        }
    }
    else /* Layer 3 LSF */
    {
        si->main_data_begin = u_mp3_decoder.u_frame_buffer->getbits(8);
        if (stereo == 1)
            si->private_bits = u_mp3_decoder.u_frame_buffer->getbits(1);
        else
            si->private_bits = u_mp3_decoder.u_frame_buffer->getbits(2);

        gr=0;

        for (ch=0; ch<stereo; ch++)
        {
            si->ch[ch].gr[gr].part2_3_length        = u_mp3_decoder.u_frame_buffer->getbits(12);
            si->ch[ch].gr[gr].big_values            = u_mp3_decoder.u_frame_buffer->getbits(9);
            si->ch[ch].gr[gr].global_gain           = u_mp3_decoder.u_frame_buffer->getbits(8);
            si->ch[ch].gr[gr].scalefac_compress     = u_mp3_decoder.u_frame_buffer->getbits(9);
            si->ch[ch].gr[gr].window_switching_flag = u_mp3_decoder.u_frame_buffer->get1bit();
            
            if (si->ch[ch].gr[gr].window_switching_flag)
            {
                si->ch[ch].gr[gr].block_type       = u_mp3_decoder.u_frame_buffer->getbits(2);
                si->ch[ch].gr[gr].mixed_block_flag = u_mp3_decoder.u_frame_buffer->get1bit();
                for (i=0; i<2; i++)
                    si->ch[ch].gr[gr].table_select[i] = u_mp3_decoder.u_frame_buffer->getbits(5);
                for (i=0; i<3; i++)
                    si->ch[ch].gr[gr].subblock_gain[i] = u_mp3_decoder.u_frame_buffer->getbits(3);
                
                /* Set region_count parameters since they are implicit in this case. */

                if (si->ch[ch].gr[gr].block_type == 0)
                {
#ifdef SOFT
                    printf("Side info bad: block_type == 0 in split block.\n");
#else
                    __lprintf("Side info bad\n");
#endif
                    exit(0);
                }
                else if (si->ch[ch].gr[gr].block_type == 2 && si->ch[ch].gr[gr].mixed_block_flag == 0)
                    si->ch[ch].gr[gr].region0_count = 8; /* MI 9; */
                else
                    si->ch[ch].gr[gr].region0_count = 7; /* MI 8; */

                si->ch[ch].gr[gr].region1_count = 20 - si->ch[ch].gr[gr].region0_count;
            }
            else
            {
                for (i=0; i<3; i++)
                    si->ch[ch].gr[gr].table_select[i] = u_mp3_decoder.u_frame_buffer->getbits(5);
                si->ch[ch].gr[gr].region0_count = u_mp3_decoder.u_frame_buffer->getbits(4);
                si->ch[ch].gr[gr].region1_count = u_mp3_decoder.u_frame_buffer->getbits(3);
                si->ch[ch].gr[gr].block_type    = 0;
            }
            
            si->ch[ch].gr[gr].scalefac_scale     = u_mp3_decoder.u_frame_buffer->get1bit();
            si->ch[ch].gr[gr].count1table_select = u_mp3_decoder.u_frame_buffer->get1bit();
        }   /* for Channel */
    }   /* Layer 3 */
}

///////////////////////////////////////////////////////////////////////////////////
//  Main Data Slot
//
//  Return the number of slots for main data of current frame
//

// Bit Rate : 1: MPEG-1, 0: MPEG-2 LSF
int bitrate[2][3][15] =
{
    {
        {0,32,48,56,64,80,96,112,128,144,160,176,192,224,256},
        {0,8,16,24,32,40,48,56,64,80,96,112,128,144,160},
        {0,8,16,24,32,40,48,56,64,80,96,112,128,144,160}
    },
    {
        {0,32,64,96,128,160,192,224,256,288,320,352,384,416,448},
        {0,32,48,56,64,80,96,112,128,160,192,224,256,320,384},
        {0,32,40,48,56,64,80,96,112,128,160,192,224,256,320}
    }
};

// Sampling Frequency : 1: MPEG-1, 0: MPEG-2 LSF
double s_freq[2][4] =
{
    {22.05, 24, 16, 0}, /* MPEG-2 LSF */
    {44.1, 48, 32, 0}   /* MPEG-1 */
};

int iii_headerinfo::main_data_slots(frame_params fr_ps)
{
    int nSlots;

    /* Frame Size */
    nSlots = (int)((144 * bitrate[fr_ps.header->version][2][fr_ps.header->bitrate_index])
                    / s_freq[fr_ps.header->version][fr_ps.header->sampling_frequency]);

    /* Side_Info Size */
    if ( fr_ps.header->version != MPEG_PHASE2_LSF )
    {   /* MPEG-1 */
        if (fr_ps.stereo == 1)
            nSlots -= 17;
        else
            nSlots -=32;
    }
    else
    {   /* MPEG-2 LSF */
        nSlots = nSlots / 2;
        if (fr_ps.stereo == 1)
            nSlots -= 9;
        else
            nSlots -=17;
    }

    /* Padding ? */
    if (fr_ps.header->padding)
        nSlots++;

    /* 32-Bits Header Info Size */
    nSlots -= 4;

    /* CRC Protected ? */
    if (fr_ps.header->error_protection)
        nSlots -= 2;

  return(nSlots);
}
