///////////////////////////////////////////////////////////////////////////
// Filename : sc_III_Huffman.cpp
// Project  : MP3 Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision :
// Comment  :
//
//

#include "MP3_Decoder.h"
#include "sc_III_Huffman.h"
#include "sc_III_Frame_buffer.h"
#include "sc_III_Decoder_Mem.h"

#include "sfBandIndex.h"

void iii_huffman::proc_huffman()
{
    int     t_version               = version.read();
    int     t_big_values            = (int)(sc_uint<9>)big_values.read();
    int     t_table_select0         = (int)(sc_uint<5>)table_select0.read();
    int     t_table_select1         = (int)(sc_uint<5>)table_select1.read();
    int     t_table_select2         = (int)(sc_uint<5>)table_select2.read();
    int     t_count1table_select    = count1table_select.read();
    int     t_sampling_frequency    = (int)(sc_uint<2>)sampling_frequency.read();
    int     t_region0_count         = (int)(sc_uint<4>)region0_count.read();
    int     t_region1_count         = (int)(sc_uint<3>)region1_count.read();
    int     t_part2_3_length        = (int)(sc_uint<12>)part2_3_length.read();
    int     t_window_switching_flag = window_switching_flag.read();
    int     t_block_type            = (int)(sc_uint<2>)block_type.read();
    int     t_ch                    = ch.read();
    int     t_gr                    = gr.read();
    unsigned long   t_part2_start   = part2_start.read();

    if (rst.read()==(sc_bit)'0')    // Async. Reset
    {
        Huffman_State = S_HUFFMAN_WAIT;
        ready = (sc_bit)'1';
    }
    else if (clk.event() && clk.read()==(sc_bit)'1')    // Clocking Rising-Edge
    {
        switch(Huffman_State.read())
        {
            case S_HUFFMAN_WAIT:
                if (start.read()==(sc_bit)'1')
                {
                    ready = (sc_bit)'0';
                    Huffman_State = S_HUFFMAN_DECODE;
                }
                break;

            case S_HUFFMAN_DECODE:
                if (start.read()==(sc_bit)'0')
                {
                    III_hufman_decode(
                                t_version,
                                t_big_values,
                                t_table_select0,
                                t_table_select1,
                                t_table_select2,
                                t_count1table_select,
                                t_sampling_frequency,
                                t_region0_count,
                                t_region1_count,
                                t_part2_3_length,
                                t_window_switching_flag,
                                t_block_type,
                                t_ch,
                                t_gr,
                                t_part2_start);

                    Huffman_State = S_HUFFMAN_DOWNLOAD;
                }
                break;

            case S_HUFFMAN_DOWNLOAD:
                Huffman_State = S_HUFFMAN_BUSY;
#ifdef _DEBUG_OUT
                {
                    FILE *fp;
                    int ss, sb;

                    fp = fopen("dbg_huffman.txt", "a+");

                    fprintf(fp, "Frame=%d/Granule=%d/Channel=%d\n",
                                    u_mp3_decoder.u_decoder_ctl->nFrame,
                                    ((u_mp3_decoder.u_decoder_ctl->gr)? 1:0),
                                    ((u_mp3_decoder.u_decoder_ctl->ch)? 1:0) );

                    for (sb=0; sb<SBLIMIT; sb++)
                        for (ss=0; ss<SSLIMIT; ss++)
                            fprintf(fp, "[%d][%2d][%2d] %8d\n", t_ch, sb, ss,
                                        u_mp3_decoder.u_decoder_mem->get_is(t_ch, sb, ss));
                    fflush(fp);
                    fclose(fp);
                }
#endif
                break;

            case S_HUFFMAN_BUSY:
                ready = (sc_bit)'1';
                Huffman_State = S_HUFFMAN_WAIT;
                break;

            default:
                Huffman_State = S_HUFFMAN_WAIT;
                break;
        }
    }
}

//////////////////////////////////////////////////////////////////////////
// Huffman Decoder for Layer III
//
//int huffman_initialized = FALSE;
#include <stdio.h>  // needed for definition of NULL
#include "sc_III_Huffman_table.h"

void iii_huffman::III_hufman_decode(
    int         version,
    int         big_values,
    int         table_select0,
    int         table_select1,
    int         table_select2,
    int         count1table_select,
    int         sampling_frequency,
    int         region0_count,
    int         region1_count,
    int         part2_3_length,
    int         window_switching_flag,
    int         block_type,
    int         ch,
    int         gr,
    int         part2_start)
{
    int i, x, y;
    int v, w;
    int region1Start;
    int region2Start;
    int sfreq;
    int currentBit, grBits;

    struct huffcodetab  *h;

    sfreq = sampling_frequency + (version * 3);

    // Find Region Boundary
    if (window_switching_flag && block_type==2) // Short Block
    {
        region1Start = 36;  /* sfb[9/3]*3=36 */
        region2Start = 576; /* No Region2 for short block case. */
    }
    else    // LONG Block
    {
        region1Start = sfBandIndex[sfreq].l[region0_count + 1];
        region2Start = sfBandIndex[sfreq].l[region0_count + region1_count + 2];
    }

    grBits     = part2_start + part2_3_length;
    currentBit = u_mp3_decoder.u_frame_buffer->hsstell();
    
    // Read bigvalues area.
    for (i=0; i<(int)(big_values*2); i+=2)
    {
        if (i<region1Start)
            h = &ht[table_select0];
        else if (i<region2Start)
            h = &ht[table_select1];
        else
            h = &ht[table_select2];

        huffman_decoder(h, &x, &y, &v, &w);

        u_mp3_decoder.u_decoder_mem->put_is( x, ch, i/SSLIMIT, i%SSLIMIT);
        u_mp3_decoder.u_decoder_mem->put_is( y, ch, (i+1)/SSLIMIT, (i+1)%SSLIMIT);
    }

    grBits     = part2_start + part2_3_length;
    currentBit = u_mp3_decoder.u_frame_buffer->hsstell();

    /* Read count1 area. */
    h = &ht[count1table_select+32];
    while ((u_mp3_decoder.u_frame_buffer->hsstell() < part2_start + part2_3_length ) && ( i < SSLIMIT*SBLIMIT ))
    {
        huffman_decoder(h, &x, &y, &v, &w);

        u_mp3_decoder.u_decoder_mem->put_is( v, ch, i/SSLIMIT, i%SSLIMIT);
        u_mp3_decoder.u_decoder_mem->put_is( w, ch, (i+1)/SSLIMIT, (i+1)%SSLIMIT);
        u_mp3_decoder.u_decoder_mem->put_is( x, ch, (i+2)/SSLIMIT, (i+2)%SSLIMIT);
        u_mp3_decoder.u_decoder_mem->put_is( y, ch, (i+3)/SSLIMIT, (i+3)%SSLIMIT);
        i += 4;
    }

    grBits     = part2_start + part2_3_length;
    currentBit = u_mp3_decoder.u_frame_buffer->hsstell();

    if (u_mp3_decoder.u_frame_buffer->hsstell() > part2_start + part2_3_length)
    {
        i -=4;
        u_mp3_decoder.u_frame_buffer->rewindNbits(
            u_mp3_decoder.u_frame_buffer->hsstell()-part2_start - part2_3_length);
    }
    
    /* Dismiss stuffing Bits */
    grBits     = part2_start + part2_3_length;
    currentBit = u_mp3_decoder.u_frame_buffer->hsstell();
    
    if ( currentBit < grBits )
        u_mp3_decoder.u_frame_buffer->hgetbits( grBits - currentBit );
    
    /* Zero out rest. */
    for (; i<SSLIMIT*SBLIMIT; i++)
        u_mp3_decoder.u_decoder_mem->put_is( 0, ch, i/SSLIMIT, i%SSLIMIT);
}

///////////////////////////////////////////////////////////////////////////
// Huffman decoder for Layer III
//   do the huffman-decoding
//   note! for counta,countb -the 4 bit value is returned in y, discard x
//
int iii_huffman::huffman_decoder(
    struct huffcodetab  *h,
    int                 *x,
    int                 *y,
    int                 *v,
    int                 *w)
{
    HUFFBITS    level = 1 << ((sizeof(HUFFBITS) * 8) - 1);
    int         point = 0;
    int         error = 1;

    if (h->val == NULL)
        return(2);

    /* table 0 needs no bits */
    if ( h->treelen == 0)
    {
        *x = *y = 0;
        return(0);
    }

    /* Lookup in Huffman table. */
    do
    {
        if (h->val[point][0]==0)   /*end of tree*/
        {
            *x = h->val[point][1] >> 4;
            *y = h->val[point][1] & 0xf;

            error = 0;
            break;
        }

        if (u_mp3_decoder.u_frame_buffer->hget1bit())
        {
            while (h->val[point][1] >= MXOFF)
                point += h->val[point][1];
            point += h->val[point][1];
        }
        else
        {
            while (h->val[point][0] >= MXOFF)
                point += h->val[point][0];
            point += h->val[point][0];
        }

        level >>= 1;
    } while (level  || (point < (int)ht->treelen) );

    /* Process sign encodings for quadruples tables. */
    if (h->tablename[0] == '3' &&
        (h->tablename[1] == '2' || h->tablename[1] == '3'))
    {
        *v = (*y>>3) & 1;
        *w = (*y>>2) & 1;
        *x = (*y>>1) & 1;
        *y = *y & 1;

        /* v, w, x and y are reversed in the bitstream.
            switch them around to make test bistream work. */

        if (*v)
            if (u_mp3_decoder.u_frame_buffer->hget1bit())
                *v = -*v;

        if (*w)
            if (u_mp3_decoder.u_frame_buffer->hget1bit())
                *w = -*w;

        if (*x)
            if (u_mp3_decoder.u_frame_buffer->hget1bit())
                *x = -*x;

        if (*y)
            if (u_mp3_decoder.u_frame_buffer->hget1bit())
                *y = -*y;
    }
    else
    {
        // Process sign and escape encodings for dual tables.

        // x and y are reversed in the test bitstream.
        //  Reverse x and y here to make test bitstream work.

        if (h->linbits)
            if ((int)(h->xlen-1) == *x)
                *x += u_mp3_decoder.u_frame_buffer->hgetbits(h->linbits);
        if (*x)
            if (u_mp3_decoder.u_frame_buffer->hget1bit())
                *x = -*x;

        if (h->linbits)
            if ((int)(h->ylen-1) == *y)
                *y += u_mp3_decoder.u_frame_buffer->hgetbits(h->linbits);

        if (*y)
            if (u_mp3_decoder.u_frame_buffer->hget1bit())
                *y = -*y;
    }

    return(error);
}


