///////////////////////////////////////////////////////////////////////////
// Filename : sc_III_OutFifo.h
// Project  : MP3 Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision :
// Comment  :
//
//
#ifndef III_OUTFIFO_INC
#define III_OUTFIFO_INC

#include "systemc.h"

enum tOutFifo_State {
    S_OUTFIFO_WAIT,
    S_OUTFIFO_DECODE,
    S_OUTFIFO_DOWNLOAD,
    S_OUTFIFO_BUSY };

class WavImage;

SC_MODULE(iii_outfifo)
{
    sc_in_clk           clk;    // Clock
    sc_in<sc_bit>       rst;    // Reset

    sc_in<sc_bit>       start;  // Out-FIFO Start
    sc_out<sc_bit>      ready;  // Out-FIFO Ready
    sc_in<sc_bit>       stereo;
    sc_in<sc_bit>       stop;

    sc_slave<> Done;

    // Process ///////////////////////////////////////////////////////////
    void proc_outfifo();

    void out_fifo( int stereo );

    WavImage *dWavImage;
    unsigned long dSampleFrames, dPcmCount, dPcmOverflowCount;


    // Globals ///////////////////////////////////////////////////////////
    sc_signal<tOutFifo_State>   OutFifo_State;  // FSM State Variable

    // Slave Threads /////////////////////////////////////////////////////
    void doneThread();

    // Constructor ///////////////////////////////////////////////////////////
    SC_CTOR(iii_outfifo)
      : dWavImage(NULL), dSampleFrames(0), dPcmCount(0), dPcmOverflowCount(0) {
        SC_METHOD(proc_outfifo);
        sensitive << clk << rst;

        SC_SLAVE( doneThread, Done );
    }
};

#endif
