///////////////////////////////////////////////////////////////////////////
// Filename : sc_III_Reorder.cpp
// Project  : MP3 Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision :
// Comment  :
//
//

#include "MP3_Decoder.h"
#include "sc_III_Reorder.h"

#include "sc_III_Decoder_Mem.h"
#include "sfBandIndex.h"

void iii_reorder::proc_reorder()
{
    int     t_sampling_freq         = (sc_uint<2>)sampling_frequency.read();
    int     t_window_switching_flag = window_switching_flag.read();
    int     t_block_type            = (sc_uint<2>)block_type.read();
    int     t_mixed_block_flag      = mixed_block_flag.read();
    int     t_version               = version.read();
    int     t_ch                    = channel.read();

    if (rst.read()==(sc_bit)'0')    // Async. Reset
    {
        Reorder_State = S_REORDER_WAIT;
        ready = (sc_bit)'1';
    }
    else if (clk.event() && clk.read()==(sc_bit)'1')    // Clocking Rising-Edge
    {
        switch(Reorder_State.read())
        {
            case S_REORDER_WAIT:
                if (start.read()==(sc_bit)'1')
                {
                    ready = (sc_bit)'0';
                    Reorder_State = S_REORDER_DECODE;
                }
                break;

            case S_REORDER_DECODE:
                if (start.read()==(sc_bit)'0')
                {
                    III_reorder (
                        t_sampling_freq,
                        t_window_switching_flag,
                        t_block_type,
                        t_mixed_block_flag,
                        t_version,
                        t_ch );

                    Reorder_State = S_REORDER_DOWNLOAD;
                }
                break;

            case S_REORDER_DOWNLOAD:
                Reorder_State = S_REORDER_BUSY;
#ifdef _DEBUG_OUT
                {
                    FILE *fp;
                    int ss, sb;

                    fp = fopen("dbg_reorder.txt", "a+");

                    fprintf(fp, "Frame=%d/Granule=%d/Channel=%d\n",
                                    u_mp3_decoder.u_decoder_ctl->nFrame,
                                    ((u_mp3_decoder.u_decoder_ctl->gr)? 1:0),
                                    ((u_mp3_decoder.u_decoder_ctl->ch)? 1:0) );

                    for (sb=0; sb<SBLIMIT; sb++)
                        for (ss=0; ss<SSLIMIT; ss++)
                            fprintf(fp, "[%d][%2d][%2d] %16.12f\n", t_ch, sb, ss,
                                        u_mp3_decoder.u_decoder_mem->get_re(t_ch, sb, ss));
                    fflush(fp);
                    fclose(fp);
                }
#endif
                break;

            case S_REORDER_BUSY:
                ready = (sc_bit)'1';
                Reorder_State = S_REORDER_WAIT;
                break;

            default:
                Reorder_State = S_REORDER_WAIT;
                break;
        }
    }
}

///////////////////////////////////////////////////////////////////////////
// Re-Ordering
//
void iii_reorder::III_reorder (
    int     sampling_frequency,
    int     window_switching_flag,
    int     block_type,
    int     mixed_block_flag,
    int     version,
    int     ch )
{
    int     sfreq;
    int     sfb, sfb_start, sfb_lines;
    int     sb, ss, window, freq, src_line, des_line;

    sfreq = sampling_frequency + (version * 3);

//  for(sb=0;sb<SBLIMIT;sb++)
//      for(ss=0;ss<SSLIMIT;ss++) 
//          u_mp3_decoder.u_decoder_mem->put_re( 0.0, ch, sb, ss);

    if (window_switching_flag && (block_type == 2))
    {
        if (mixed_block_flag)
        {
            // NO REORDER FOR LOW 2 SUBBANDS
            for (sb=0 ; sb < 2 ; sb++)
                for (ss=0 ; ss < SSLIMIT ; ss++)
                    u_mp3_decoder.u_decoder_mem->put_re(
                            u_mp3_decoder.u_decoder_mem->get_lr( ch, sb, ss), ch, sb, ss);

            // REORDERING FOR REST SWITCHED SHORT
            sfb       = 3;
            sfb_start = sfBandIndex[sfreq].s[3];
            sfb_lines = sfBandIndex[sfreq].s[4] - sfb_start;

            for(; sfb<13; )
            {
                for(window=0; window<3; window++)
                {
                    for(freq=0; freq<sfb_lines; freq++)
                    {
                        src_line = sfb_start*3 + window*sfb_lines + freq; 
                        des_line = (sfb_start*3) + window + (freq*3);

                        u_mp3_decoder.u_decoder_mem->put_re(
                                u_mp3_decoder.u_decoder_mem->get_lr( ch, (src_line/SSLIMIT), (src_line%SSLIMIT)),
                                ch, (des_line/SSLIMIT), (des_line%SSLIMIT) );
                    }
                }

                sfb++;
                sfb_start = sfBandIndex[sfreq].s[sfb];
                sfb_lines = sfBandIndex[sfreq].s[sfb+1] - sfb_start;
            }
        } 
        else    // Pure SHORT
        {
            sfb       = 0;
            sfb_start = 0;
            sfb_lines = sfBandIndex[sfreq].s[1];

            for(; sfb < 13; )
            {
                for(window=0; window<3; window++)
                {
                    for(freq=0;freq<sfb_lines;freq++)
                    {
                        src_line = sfb_start*3 + window*sfb_lines + freq; 
                        des_line = (sfb_start*3) + window + (freq*3);

                        u_mp3_decoder.u_decoder_mem->put_re(
                                u_mp3_decoder.u_decoder_mem->get_lr( ch, (src_line/SSLIMIT), (src_line%SSLIMIT)),
                                ch, (des_line/SSLIMIT), (des_line%SSLIMIT) );
                    }
                }

                sfb++;
                sfb_start = sfBandIndex[sfreq].s[sfb];
                sfb_lines = sfBandIndex[sfreq].s[sfb+1] - sfb_start;
            }
        }
    }
    else    // LONG
    {
        for (sb=0 ; sb < SBLIMIT ; sb++)
            for (ss=0 ; ss < SSLIMIT ; ss++) 
                u_mp3_decoder.u_decoder_mem->put_re(
                    u_mp3_decoder.u_decoder_mem->get_lr( ch, sb, ss), ch, sb, ss);
    }
}
