///////////////////////////////////////////////////////////////////////////
// Filename : sc_III_ScaleFactor.cpp
// Project  : MP3 Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision :
// Comment  :
//
//

#include "MP3_Decoder.h"
#include "sc_III_ScaleFactor.h"
#include "sc_III_Frame_buffer.h"
#include "sc_III_Decoder_Mem.h"

void iii_scalefactor::proc_scale()
{
    int     t_scalefac_compress     = (int)(sc_uint<9>)scalefac_compress.read();
    int     t_scfsi[4];
    int     t_block_type            = (int)(sc_uint<2>)block_type.read();
    int     t_window_switching_flag = window_switching_flag.read();
    int     t_mixed_block_flag      = mixed_block_flag.read();

    int     t_ch                = ch.read();
    int     t_gr                = gr.read();
    int     t_version           = version.read();
    int     t_preflag           = preflag.read();
    int     t_mode_ext          = (int)(sc_uint<2>)mode_ext.read();

    if (rst.read()==(sc_bit)'0')    // Async. Reset
    {
        part2_start = (long)0;
        preflag_out = (bool)false;

        Scale_State = S_SCALE_WAIT;
        ready = (sc_bit)'1';
    }
    else if (clk.event() && clk.read()==(sc_bit)'1')    // Clocking Rising-Edge
    {
        switch(Scale_State.read())
        {
            case S_SCALE_WAIT:
                if (start.read()==(sc_bit)'1')
                {
                    ready = (sc_bit)'0';
                    Scale_State = S_SCALE_DECODE;
                }
                break;

            case S_SCALE_DECODE:
                if (start.read()==(sc_bit)'0')
                {
                    t_scfsi[0] = scfsi0.read();
                    t_scfsi[1] = scfsi1.read();
                    t_scfsi[2] = scfsi2.read();
                    t_scfsi[3] = scfsi3.read();

                    // PART2 START for Huffman decoding
                    part2_start = u_mp3_decoder.u_frame_buffer->hsstell();

                    if (t_version != MPEG_PHASE2_LSF)
                    {
                        III_get_scale_factors(  // MPEG-1 Scale Factor
                                                t_scalefac_compress,
                                                t_scfsi,
                                                t_block_type,
                                                t_window_switching_flag,
                                                t_mixed_block_flag,
                                                t_gr,
                                                t_ch);
                    }
                    else
                    {
                        t_preflag = (bool) III_get_LSF_scale_factors(// MPEG-2 Scale Factor
                                                t_preflag,
                                                t_scalefac_compress,
                                                t_mode_ext,
                                                t_block_type,
                                                t_mixed_block_flag,
                                                t_gr,
                                                t_ch);
                    }
                    // Preflag //////////////////////
                    preflag_out = (bool)t_preflag;

                    Scale_State = S_SCALE_DOWNLOAD;
                }
                break;

            case S_SCALE_DOWNLOAD:
                Scale_State = S_SCALE_BUSY;
#ifdef _DEBUG_OUT
                {
                    FILE *fp;
                    int cb, win;

                    fp = fopen("dbg_scalefact.txt", "a+");

                    fprintf(fp, "Frame=%d/Granule=%d/Channel=%d\n",
                                    u_mp3_decoder.u_decoder_ctl->nFrame,
                                    ((u_mp3_decoder.u_decoder_ctl->gr)? 1:0),
                                    ((u_mp3_decoder.u_decoder_ctl->ch)? 1:0) );

                    for (cb=0; cb<23; cb++)
                        fprintf(fp, "l[%d][%2d] %8d\n", t_ch, cb,
                            u_mp3_decoder.u_decoder_mem->get_scalefac(t_ch, 0, cb, 0) );

                    for (win=0; win<3; win++)
                        for (cb=0; cb<13; cb++)
                            fprintf(fp, "s[%d][%2d][%2d] %8d\n", t_ch, win, cb,
                                u_mp3_decoder.u_decoder_mem->get_scalefac(t_ch, win, cb, 1) );

                    fflush(fp);
                    fclose(fp);
                }
#endif
                break;

            case S_SCALE_BUSY:
                ready = (sc_bit)'1';
                Scale_State = S_SCALE_WAIT;
                break;

            default:
                Scale_State = S_SCALE_WAIT;
                break;
        }
    }
}

///////////////////////////////////////////////////////////////////////////
// Get MPEG-1 Scale Factor of Layer III
//
void iii_scalefactor::III_get_scale_factors(
    int             scalefac_compress,
    int             *scfsi,
    int             block_type,
    int             window_switching_flag,
    int             mixed_block_flag,
    int             gr,
    int             ch)
{
    static struct {
        int l[5];
        int s[3];
    } sfbtable = {
        {0, 6, 11, 16, 21},
        {0, 6, 12}
    };

    static int slen[2][16] = {
        {0, 0, 0, 0, 3, 1, 1, 1, 2, 2, 2, 3, 3, 3, 4, 4},
        {0, 1, 2, 3, 0, 1, 2, 3, 1, 2, 3, 1, 2, 3, 2, 3}
    };

    int     i;
    int     sfb;
    int     window;

    if (window_switching_flag && block_type == 2)
    { 
        if (mixed_block_flag)   /* MIXED Block */
        {
            for (sfb = 0; sfb < 8; sfb++)
                u_mp3_decoder.u_decoder_mem->put_scalefac(
                    u_mp3_decoder.u_frame_buffer->hgetbits(slen[0][scalefac_compress]), ch, 0, sfb, 0);
            for (sfb = 3; sfb < 6; sfb++)
                for (window=0; window<3; window++)
                    u_mp3_decoder.u_decoder_mem->put_scalefac(
                        u_mp3_decoder.u_frame_buffer->hgetbits(slen[0][scalefac_compress]), ch, window, sfb, 1);
            for (sfb = 6; sfb < 12; sfb++)
                for (window=0; window<3; window++)
                    u_mp3_decoder.u_decoder_mem->put_scalefac(
                        u_mp3_decoder.u_frame_buffer->hgetbits(slen[1][scalefac_compress]), ch, window, sfb, 1);
            for (sfb=12,window=0; window<3; window++)
                u_mp3_decoder.u_decoder_mem->put_scalefac( 0, ch, window, sfb, 1);
        }
        else /* SHORT Block */
        {
            for (i=0; i<2; i++) 
                for (sfb = sfbtable.s[i]; sfb < sfbtable.s[i+1]; sfb++)
                    for (window=0; window<3; window++)
                        u_mp3_decoder.u_decoder_mem->put_scalefac(
                            u_mp3_decoder.u_frame_buffer->hgetbits(slen[i][scalefac_compress]), ch, window, sfb, 1);
            for (sfb=12,window=0; window<3; window++)
                u_mp3_decoder.u_decoder_mem->put_scalefac( 0, ch, window, sfb, 1);
        }
    }          
    else /* LONG Block */
    {
        for (i=0; i<4; i++)
        {
            if (scfsi[i]==0 || gr==0)
                for (sfb = sfbtable.l[i]; sfb < sfbtable.l[i+1]; sfb++)
                    u_mp3_decoder.u_decoder_mem->put_scalefac(
                        u_mp3_decoder.u_frame_buffer->hgetbits(slen[(i<2)?0:1][scalefac_compress]), ch, 0, sfb, 0);
        }
        u_mp3_decoder.u_decoder_mem->put_scalefac( 0, ch, 0, 22, 0);
    }
}

///////////////////////////////////////////////////////////////////////////
// Get MPEG-2 Scale Factor of Layer III
//
int iii_scalefactor::III_get_LSF_scale_factors(
    int             preflag,
    int             scalefac_comp,
    int             mode_ext,
    int             block_type,
    int             mixed_block_flag,
    int             gr,
    int             ch )
{
    static unsigned nr_of_sfb_block[6][3][4] = {
                            {{ 6,  5, 5, 5},{ 9, 9, 9, 9},{ 6, 9, 9,9}},
                            {{ 6,  5, 7, 3},{ 9, 9,12, 6},{ 6, 9,12,6}},
                            {{11, 10, 0, 0},{18,18, 0, 0},{15,18, 0,0}},
                            {{ 7,  7, 7, 0},{12,12,12, 0},{ 6,15,12,0}},
                            {{ 6,  6, 6, 3},{12, 9, 9, 6},{ 6,12, 9,6}},
                            {{ 8,  8, 5, 0},{15,12, 9, 0},{ 6,18, 9,0}}};
    static unsigned scalefac_buffer[54];

    int         i, j, k;
    int         sfb, window;
    int         blocktypenumber, blocknumber;
    unsigned    int_scalefac_comp, new_slen[4];

    /* Get Scale Data */
    if ((block_type==2) && (mixed_block_flag==0))       blocktypenumber = 1;
    else if ((block_type==2) && (mixed_block_flag==1))  blocktypenumber = 2;
    else                                                blocktypenumber = 0;

    if(!(((mode_ext==1) || (mode_ext==3)) && (ch==1)))
    {
        if(scalefac_comp < 400)
        {
            new_slen[0] = (scalefac_comp >> 4) / 5 ;
            new_slen[1] = (scalefac_comp >> 4) % 5 ;
            new_slen[2] = (scalefac_comp % 16) >> 2 ;
            new_slen[3] = (scalefac_comp % 4);
            preflag = 0;
            blocknumber = 0;
        }
        else if( scalefac_comp  < 500)
        {
            new_slen[0] = ((scalefac_comp - 400 )  >> 2) / 5 ;
            new_slen[1] = ((scalefac_comp - 400) >> 2) % 5 ;
            new_slen[2] = (scalefac_comp - 400 ) % 4 ;
            new_slen[3] = 0;
            preflag = 0;
            blocknumber = 1;

        }
        else if( scalefac_comp  < 512)
        {
            new_slen[0] = (scalefac_comp - 500 ) / 3 ;
            new_slen[1] = (scalefac_comp - 500)  % 3 ;
            new_slen[2] = 0 ;
            new_slen[3] = 0;
            preflag = 1;
            blocknumber = 2;
        }
    }

    if((((mode_ext==1) || (mode_ext==3)) && (ch==1)))
    {
        /*   intensity_scale = scalefac_comp %2; */
        int_scalefac_comp = scalefac_comp >> 1;
    
        if(int_scalefac_comp  < 180)
        {
            new_slen[0] = int_scalefac_comp  / 36 ;
            new_slen[1] = (int_scalefac_comp % 36 ) / 6 ;
            new_slen[2] = (int_scalefac_comp % 36) % 6;
            new_slen[3] = 0;
            preflag = 0;
            blocknumber = 3;
        }
        else if( int_scalefac_comp  < 244)
        {
            new_slen[0] = ((int_scalefac_comp - 180 )  % 64 ) >> 4 ;
            new_slen[1] = ((int_scalefac_comp - 180) % 16) >> 2 ;
            new_slen[2] = (int_scalefac_comp - 180 ) % 4 ;
            new_slen[3] = 0;
            preflag = 0;
            blocknumber = 4;
        }
        else if( int_scalefac_comp  < 255)
        {
            new_slen[0] = (int_scalefac_comp - 244 ) / 3 ;
            new_slen[1] = (int_scalefac_comp - 244 )  % 3 ;
            new_slen[2] = 0 ;
            new_slen[3] = 0;
            preflag = 0;
            blocknumber = 5;
        }
    }
     
    for(i=0;i< 45;i++) scalefac_buffer[i] = 0;

    k = 0;
    for(i = 0;i < 4;i++)
    { 
        for(j = 0; j < nr_of_sfb_block[blocknumber][blocktypenumber][i]; j++)
        {
            if(new_slen[i] == 0)
                scalefac_buffer[k] = 0;
            else
                scalefac_buffer[k] = u_mp3_decoder.u_frame_buffer->hgetbits(new_slen[i]);
            k++;
        }
    }

    /* LSF Scale Factor */
    k = 0;
    if (window_switching_flag && (block_type == 2))
    { 
        if (mixed_block_flag) /* MIXED */
        {
            for (sfb = 0; sfb < 8; sfb++)
            {
                u_mp3_decoder.u_decoder_mem->put_scalefac(scalefac_buffer[k], ch, 0, sfb, 0);
                k++;
            } 
            for (sfb = 3; sfb < 12; sfb++)
                for (window=0; window<3; window++)
                {
                    u_mp3_decoder.u_decoder_mem->put_scalefac(scalefac_buffer[k], ch, window, sfb, 1);
                    k++;
                }
            for (sfb=12,window=0; window<3; window++)
                u_mp3_decoder.u_decoder_mem->put_scalefac(0, ch, window, sfb, 1);
        }
        else /* SHORT*/
        {
            for (sfb = 0; sfb < 12; sfb++)
                for (window=0; window<3; window++)
                {
                    u_mp3_decoder.u_decoder_mem->put_scalefac(scalefac_buffer[k], ch, window, sfb, 1);
                    k++;
                }
            for (sfb=12,window=0; window<3; window++)
                u_mp3_decoder.u_decoder_mem->put_scalefac(0, ch, window, sfb, 1);
        }
    }          
    else /* LONG Block */
    {
        for (sfb = 0; sfb < 21; sfb++)
        {
            u_mp3_decoder.u_decoder_mem->put_scalefac(scalefac_buffer[k], ch, 0, sfb, 0);
            k++;
        }
        u_mp3_decoder.u_decoder_mem->put_scalefac(0, ch, 0, sfb, 0);
    }

    return(preflag);
}
