///////////////////////////////////////////////////////////////////////////
// Filename : sc_MP3_Decoder.h
// Project  : MP3 Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision :
// Comment  : 
//
//
#ifndef MP3_DECODER_INC
#define MP3_DECODER_INC

#include "systemc.h"
#include "sc_III_Decoder_ctl.h"
#include "sc_III_Frame_buffer.h"
#include "sc_III_Decoder_Mem.h"
#include "sc_III_HeaderInfo.h"
#include "sc_III_ScaleFactor.h"
#include "sc_III_Huffman.h"
#include "sc_III_Dequantizer.h"
#include "sc_III_Stereo.h"
#include "sc_III_Reorder.h"
#include "sc_III_Antialias.h"
#include "sc_III_Hybrid.h"
#include "sc_III_Subband.h"
#include "sc_III_OutFifo.h"

#ifdef _DEBUG_VCD
#include "monitor_header.h"
#include "monitor_decoder_ctl.h"
#include "monitor_scale.h"
#endif

SC_MODULE(mp3_decoder)
{
    // In/Out Ports //////////////////////////////////////////////////////
    sc_in_clk           clk;    // Clock
    sc_in<sc_bit>       rst;    // Reset
    sc_in<sc_bit>       stop;   // STOP Button
    sc_link_mp<>        done;

    sc_in<sc_bit>       mp3_ack;
    sc_out<sc_bit>      mp3_req;
    sc_in<sc_bv<8> >    mp3_data;

    // Signals /////////////////////////////////////////////////////////////
    public:
    sc_signal<sc_bit>       hdr_ready;          // Header Ready
    sc_signal<sc_bit>       hdr_start;          // Header Start
    sc_signal<sc_bit>       scl_ready;          // Scale Factor Ready
    sc_signal<sc_bit>       scl_start;          // Scale Factor Start
    sc_signal<sc_bit>       huff_ready;         // Huffman Ready
    sc_signal<sc_bit>       huff_start;         // Huffman Start
    sc_signal<sc_bit>       deq_ready;          // Dequantizer Ready
    sc_signal<sc_bit>       deq_start;          // Dequantizer Start
    sc_signal<sc_bit>       stereo_ready;       // Stereo Ready
    sc_signal<sc_bit>       stereo_start;       // Stereo Start
    sc_signal<sc_bit>       reorder_ready;      // Reorder Ready
    sc_signal<sc_bit>       reorder_start;      // Reorder Start
    sc_signal<sc_bit>       antialias_ready;    // Antialias Ready
    sc_signal<sc_bit>       antialias_start;    // Antialias Start
    sc_signal<sc_bit>       hybrid_ready;       // Hybrid Filter Bank Ready
    sc_signal<sc_bit>       hybrid_start;       // Hybrid Filter Bank Start
    sc_signal<sc_bit>       subband_ready;      // Subband Synthesis Filter Ready
    sc_signal<sc_bit>       subband_start;      // Subband Synthesis Filter Start
    sc_signal<sc_bit>       pcmout_start;

    sc_signal<sc_bit>       m_init_done;        // Frame FIFO Initializer
    sc_signal<sc_bit>       mem_init;
    sc_signal<sc_bit>       stream_full;        // Frame FIFO Fill
    sc_signal<sc_bit>       fill_stream;

    sc_signal<sc_bit>       channel;
    sc_signal<sc_bit>       granule;

    // Header Frame Params
    sc_signal<sc_bit>       stereo;
    sc_signal<sc_bv<4> >    bitrate_index;
    sc_signal<sc_bv<2> >    mode;
    sc_signal<sc_bv<2> >    mode_ext;
    sc_signal<sc_bit>       version;
    sc_signal<sc_bv<2> >    sampling_frequency;
    // Side Information
    sc_signal<sc_bv<9> >    big_values;
    sc_signal<sc_bv<2> >    block_type;
    sc_signal<sc_bit>       count1table_select;
    sc_signal<sc_bv<8> >    global_gain;
    sc_signal<sc_bv<3> >    subblock_gain0;
    sc_signal<sc_bv<3> >    subblock_gain1;
    sc_signal<sc_bv<3> >    subblock_gain2;
    sc_signal<sc_bit>       mixed_block_flag;
    sc_signal<sc_bv<12> >   part2_3_length;
    sc_signal<sc_bit>       preflag;
    sc_signal<sc_bit>       preflag_out;
    sc_signal<sc_bv<4>  >   region0_count;
    sc_signal<sc_bv<3>  >   region1_count;
    sc_signal<sc_bv<9> >    scalefac_compress;
    sc_signal<sc_bit>       scalefac_scale;
    sc_signal<sc_bit>       scfsi0;
    sc_signal<sc_bit>       scfsi1;
    sc_signal<sc_bit>       scfsi2;
    sc_signal<sc_bit>       scfsi3;
    sc_signal<sc_bv<5> >    table_select0;
    sc_signal<sc_bv<5> >    table_select1;
    sc_signal<sc_bv<5> >    table_select2;
    sc_signal<sc_bit>       window_switching_flag;

    sc_signal<unsigned long>    part2_start;

    // Declare Sub-Modules (Components) //////////////////////////////////
    public:
    decoder_ctl     *u_decoder_ctl;
    frame_buffer    *u_frame_buffer;
    decoder_mem     *u_decoder_mem;
    iii_headerinfo  *u_header;
    iii_scalefactor *u_scale;
    iii_huffman     *u_huffman;
    iii_dequantizer *u_dequantizer;
    iii_stereo      *u_stereo;
    iii_reorder     *u_reorder;
    iii_antialias   *u_antialias;
    iii_hybrid      *u_hybrid;
    iii_subband     *u_subband;
    iii_outfifo     *u_outfifo;

#ifdef _DEBUG_VCD
    monitor_decoder_ctl *u_monitor_decoder_ctl;
    monitor_header      *u_monitor_header;
    monitor_scale       *u_monitor_scale;
#endif

    // Process ///////////////////////////////////////////////////////////
    void proc_mp3_decoder();

    // PCM Output Filename ///////////////////////////////////////////////
    char pcm_filename[128];

    // Constructor ///////////////////////////////////////////////////////////
    SC_CTOR(mp3_decoder)
    {
        // UUT : decoder_ctl /////////////////////////////////////////////////
        u_decoder_ctl = new decoder_ctl("u_decoder_ctl");
        u_decoder_ctl->clk(clk);
        u_decoder_ctl->rst(rst);
        u_decoder_ctl->hdr_ready(hdr_ready);                // Header Ready
        u_decoder_ctl->hdr_start(hdr_start);                // Header Start
        u_decoder_ctl->scl_ready(scl_ready);                // Scale Factor Ready
        u_decoder_ctl->scl_start(scl_start);                // Scale Factor Start
        u_decoder_ctl->huff_ready(huff_ready);              // Huffman Ready
        u_decoder_ctl->huff_start(huff_start);              // Huffman Start
        u_decoder_ctl->deq_ready(deq_ready);                // Dequantizer Ready
        u_decoder_ctl->deq_start(deq_start);                // Dequantizer Start
        u_decoder_ctl->stereo_ready(stereo_ready);          // Stereo Ready
        u_decoder_ctl->stereo_start(stereo_start);          // Stereo Start
        u_decoder_ctl->reorder_ready(reorder_ready);        // Reorder Ready
        u_decoder_ctl->reorder_start(reorder_start);        // Reorder Start
        u_decoder_ctl->antialias_ready(antialias_ready);    // Antialias Ready
        u_decoder_ctl->antialias_start(antialias_start);    // Antialias Start
        u_decoder_ctl->hybrid_ready(hybrid_ready);          // Hybrid Filter Bank Ready
        u_decoder_ctl->hybrid_start(hybrid_start);          // Hybrid Filter Bank Start
        u_decoder_ctl->subband_ready(subband_ready);        // Subband Synthesis Filter Ready
        u_decoder_ctl->subband_start(subband_start);        // Subband Synthesis Filter Start
        u_decoder_ctl->pcmout_start(pcmout_start);
        u_decoder_ctl->stereo(stereo);
        u_decoder_ctl->mode(mode);
        u_decoder_ctl->version(version);
        u_decoder_ctl->block_type(block_type);
        u_decoder_ctl->channel(channel);
        u_decoder_ctl->granule(granule);
        u_decoder_ctl->m_init_done(m_init_done);            // Frame FIFO Initializer
        u_decoder_ctl->mem_init(mem_init);
        u_decoder_ctl->stream_full(stream_full);            // Frame FIFO Fill
        u_decoder_ctl->fill_stream(fill_stream);
        u_decoder_ctl->stop(stop);                          // STOP Button
        u_decoder_ctl->Done(done);                          // STOP Button
    
        // Frame Buffer ////////////////////////////////////////////////////
        u_frame_buffer = new frame_buffer("u_frame_buffer");
        u_frame_buffer->clk(clk);
        u_frame_buffer->rst(rst);
        u_frame_buffer->fill_stream(fill_stream);
        u_frame_buffer->stream_full(stream_full);
        u_frame_buffer->mp3_ack(mp3_ack);
        u_frame_buffer->mp3_req(mp3_req);
        u_frame_buffer->mp3_data(mp3_data);

        // Decoder Memory ///////////////////////////////////////////////////
        u_decoder_mem = new decoder_mem("u_decoder_mem");
        u_decoder_mem->clk(clk);
        u_decoder_mem->rst(rst);
        u_decoder_mem->mem_init(mem_init);
        u_decoder_mem->m_init_done(m_init_done);

        // Header //////////////////////////////////////////////////////////
        u_header = new iii_headerinfo("u_header");
        u_header->clk(clk);
        u_header->rst(rst);
        u_header->ready(hdr_ready);
        u_header->start(hdr_start);
        u_header->ch(channel);
        u_header->gr(granule);
        u_header->big_values(big_values);
        u_header->block_type(block_type);
        u_header->count1table_select(count1table_select);
        u_header->global_gain(global_gain);
        u_header->subblock_gain0(subblock_gain0);
        u_header->subblock_gain1(subblock_gain1);
        u_header->subblock_gain2(subblock_gain2);
        u_header->mixed_block_flag(mixed_block_flag);
        u_header->part2_3_length(part2_3_length);
        u_header->preflag(preflag);
        u_header->region0_count(region0_count);
        u_header->region1_count(region1_count);
        u_header->scalefac_compress(scalefac_compress);
        u_header->scalefac_scale(scalefac_scale);
        u_header->scfsi0(scfsi0);
        u_header->scfsi1(scfsi1);
        u_header->scfsi2(scfsi2);
        u_header->scfsi3(scfsi3);
        u_header->table_select0(table_select0);
        u_header->table_select1(table_select1);
        u_header->table_select2(table_select2);
        u_header->window_switching_flag(window_switching_flag);
        u_header->stereo(stereo);
        u_header->bitrate_index(bitrate_index);
        u_header->mode(mode);
        u_header->mode_ext(mode_ext);
        u_header->version(version);
        u_header->sampling_frequency(sampling_frequency);
    
        // Scale Factor ////////////////////////////////////////////////////
        u_scale =  new iii_scalefactor("u_scale");
        u_scale->clk(clk);
        u_scale->rst(rst);
        u_scale->ready(scl_ready);
        u_scale->start(scl_start);
        u_scale->ch(channel);
        u_scale->gr(granule);
        u_scale->mode_ext(mode_ext);
        u_scale->scalefac_compress(scalefac_compress);
        u_scale->scalefac_scale(scalefac_scale);
        u_scale->block_type(block_type);
        u_scale->mixed_block_flag(mixed_block_flag);
        u_scale->window_switching_flag(window_switching_flag);
        u_scale->scfsi0(scfsi0);
        u_scale->scfsi1(scfsi1);
        u_scale->scfsi2(scfsi2);
        u_scale->scfsi3(scfsi3);
        u_scale->version(version);
        u_scale->preflag(preflag);
        u_scale->preflag_out(preflag_out);
        u_scale->part2_start(part2_start);
    
        // Huffman /////////////////////////////////////////////////////////
        u_huffman = new iii_huffman("u_huffman");
        u_huffman->clk(clk);
        u_huffman->rst(rst);
        u_huffman->ready(huff_ready);
        u_huffman->start(huff_start);
        u_huffman->ch(channel);
        u_huffman->gr(granule);
        u_huffman->big_values(big_values);
        u_huffman->count1table_select(count1table_select);
        u_huffman->part2_3_length(part2_3_length);
        u_huffman->region0_count(region0_count);
        u_huffman->region1_count(region1_count);
        u_huffman->sampling_frequency(sampling_frequency);
        u_huffman->table_select0(table_select0);
        u_huffman->table_select1(table_select1);
        u_huffman->table_select2(table_select2);
        u_huffman->version(version);
        u_huffman->part2_start(part2_start);
        u_huffman->block_type(block_type);
        u_huffman->window_switching_flag(window_switching_flag);
    
        // Dequantizer /////////////////////////////////////////////////////
        u_dequantizer = new iii_dequantizer("u_dequantizer");
        u_dequantizer->clk(clk);
        u_dequantizer->rst(rst);
        u_dequantizer->ready(deq_ready);
        u_dequantizer->start(deq_start);
        u_dequantizer->sampling_frequency(sampling_frequency);
        u_dequantizer->version(version);
        u_dequantizer->window_switching_flag(window_switching_flag);
        u_dequantizer->block_type(block_type);
        u_dequantizer->mixed_block_flag(mixed_block_flag);
        u_dequantizer->global_gain(global_gain);
        u_dequantizer->subblock_gain0(subblock_gain0);
        u_dequantizer->subblock_gain1(subblock_gain1);
        u_dequantizer->subblock_gain2(subblock_gain2);
        u_dequantizer->scalefac_scale(scalefac_scale);
        u_dequantizer->preflag(preflag_out);
        u_dequantizer->channel(channel);
    
        // Stereo //////////////////////////////////////////////////////////
        u_stereo = new iii_stereo("u_stereo");
        u_stereo->clk(clk);
        u_stereo->rst(rst);
        u_stereo->ready(stereo_ready);
        u_stereo->start(stereo_start);
        u_stereo->stereo(stereo);
        u_stereo->mode(mode);
        u_stereo->mode_ext(mode_ext);
        u_stereo->scalefac_compress(scalefac_compress);
        u_stereo->sampling_freq(sampling_frequency);
        u_stereo->version(version);
        u_stereo->block_type(block_type);
        u_stereo->window_switching_flag(window_switching_flag);
        u_stereo->mixed_block_flag(mixed_block_flag);
    
        // Reorder ////////////////////////////////////////////////////////
        u_reorder = new iii_reorder("u_reorder");
        u_reorder->clk(clk);
        u_reorder->rst(rst);
        u_reorder->ready(reorder_ready);
        u_reorder->start(reorder_start);
        u_reorder->sampling_frequency(sampling_frequency);
        u_reorder->window_switching_flag(window_switching_flag);
        u_reorder->block_type(block_type);
        u_reorder->mixed_block_flag(mixed_block_flag);
        u_reorder->version(version);
        u_reorder->channel(channel);
    
        // Antialias ///////////////////////////////////////////////////////
        u_antialias = new iii_antialias("u_antialias");
        u_antialias->clk(clk);
        u_antialias->rst(rst);
        u_antialias->ready(antialias_ready);
        u_antialias->start(antialias_start);
        u_antialias->mixed_block_flag(mixed_block_flag);
        u_antialias->window_switching_flag(window_switching_flag);
        u_antialias->block_type(block_type);
        u_antialias->channel(channel);
    
        // Hybrid//////////////////////////////////////////////////////////
        u_hybrid = new iii_hybrid("u_hybrid");
        u_hybrid->clk(clk);
        u_hybrid->rst(rst);
        u_hybrid->ready(hybrid_ready);
        u_hybrid->start(hybrid_start);
        u_hybrid->mixed_block_flag(mixed_block_flag);
        u_hybrid->window_switching_flag(window_switching_flag);
        u_hybrid->block_type(block_type);
        u_hybrid->channel(channel);
    
        // Subband /////////////////////////////////////////////////////////
        u_subband = new iii_subband("u_subband");
        u_subband->clk(clk);
        u_subband->rst(rst);
        u_subband->ready(subband_ready);
        u_subband->start(subband_start);
        u_subband->stereo(stereo);

        // Out_Fifo ////////////////////////////////////////////////////////
        u_outfifo = new iii_outfifo("u_outfifo");
        u_outfifo->clk(clk);
        u_outfifo->rst(rst);
        u_outfifo->ready(subband_ready);
        u_outfifo->start(subband_start);
        u_outfifo->stereo(stereo);
        u_outfifo->stop(stop);
        u_outfifo->Done(done);

        // Process Method //////////////////////////////////////////////////
        SC_METHOD(proc_mp3_decoder);
        sensitive << clk << rst;

#ifdef _DEBUG_VCD
        // Monitor Decoder_Ctl //////////////////////////////////////////////
        u_monitor_decoder_ctl = new monitor_decoder_ctl("u_monitor_decoder_ctl");
        u_monitor_decoder_ctl->clk(clk);                        // Clock
        u_monitor_decoder_ctl->rst(rst);                        // Reset
        u_monitor_decoder_ctl->hdr_ready(hdr_ready);            // Header Ready
        u_monitor_decoder_ctl->hdr_start(hdr_start);            // Header Start
        u_monitor_decoder_ctl->scl_ready(scl_ready);            // Scale Factor Ready
        u_monitor_decoder_ctl->scl_start(scl_start);            // Scale Factor Start
        u_monitor_decoder_ctl->huff_ready(huff_ready);          // Huffman Ready
        u_monitor_decoder_ctl->huff_start(huff_start);          // Huffman Start
        u_monitor_decoder_ctl->deq_ready(deq_ready);            // Dequantizer Ready
        u_monitor_decoder_ctl->deq_start(deq_start);            // Dequantizer Start
        u_monitor_decoder_ctl->stereo_ready(stereo_ready);      // Stereo Ready
        u_monitor_decoder_ctl->stereo_start(stereo_start);      // Stereo Start
        u_monitor_decoder_ctl->reorder_ready(reorder_ready);    // Reorder Ready
        u_monitor_decoder_ctl->reorder_start(reorder_start);    // Reorder Start
        u_monitor_decoder_ctl->antialias_ready(antialias_ready);// Antialias Ready
        u_monitor_decoder_ctl->antialias_start(antialias_start);// Antialias Start
        u_monitor_decoder_ctl->hybrid_ready(hybrid_ready);      // Hybrid Filter Bank Ready
        u_monitor_decoder_ctl->hybrid_start(hybrid_start);      // Hybrid Filter Bank Start
        u_monitor_decoder_ctl->subband_ready(subband_ready);    // Subband Synthesis Filter Ready
        u_monitor_decoder_ctl->subband_start(subband_start);    // Subband Synthesis Filter Start
        u_monitor_decoder_ctl->pcmout_start(pcmout_start);
        u_monitor_decoder_ctl->stereo(stereo);
        u_monitor_decoder_ctl->mode(mode);
        u_monitor_decoder_ctl->version(version);
        u_monitor_decoder_ctl->block_type(block_type);
        u_monitor_decoder_ctl->channel(channel);
        u_monitor_decoder_ctl->granule(granule);
        u_monitor_decoder_ctl->m_init_done(m_init_done);        // Frame FIFO Initializer
        u_monitor_decoder_ctl->mem_init(mem_init);
        u_monitor_decoder_ctl->stream_full(stream_full);        // Frame FIFO Fill
        u_monitor_decoder_ctl->fill_stream(fill_stream);
        u_monitor_decoder_ctl->stop(stop);                      // STOP Button

        // Monitor Header //////////////////////////////////////////////////
        u_monitor_header = new monitor_header("u_monitor_header");
        u_monitor_header->clk(clk);
        u_monitor_header->rst(rst);
        u_monitor_header->ready(hdr_ready);
        u_monitor_header->start(hdr_start);
        u_monitor_header->ch(channel);
        u_monitor_header->gr(granule);
        u_monitor_header->big_values(big_values);
        u_monitor_header->block_type(block_type);
        u_monitor_header->count1table_select(count1table_select);
        u_monitor_header->global_gain(global_gain);
        u_monitor_header->subblock_gain0(subblock_gain0);
        u_monitor_header->subblock_gain1(subblock_gain1);
        u_monitor_header->subblock_gain2(subblock_gain2);
        u_monitor_header->mixed_block_flag(mixed_block_flag);
        u_monitor_header->part2_3_length(part2_3_length);
        u_monitor_header->preflag(preflag);
        u_monitor_header->region0_count(region0_count);
        u_monitor_header->region1_count(region1_count);
        u_monitor_header->scalefac_compress(scalefac_compress);
        u_monitor_header->scalefac_scale(scalefac_scale);
        u_monitor_header->scfsi0(scfsi0);
        u_monitor_header->scfsi1(scfsi1);
        u_monitor_header->scfsi2(scfsi2);
        u_monitor_header->scfsi3(scfsi3);
        u_monitor_header->table_select0(table_select0);
        u_monitor_header->table_select1(table_select1);
        u_monitor_header->table_select2(table_select2);
        u_monitor_header->window_switching_flag(window_switching_flag);
        u_monitor_header->stereo(stereo);
        u_monitor_header->bitrate_index(bitrate_index);
        u_monitor_header->mode(mode);
        u_monitor_header->mode_ext(mode_ext);
        u_monitor_header->version(version);
        u_monitor_header->sampling_frequency(sampling_frequency);

        // Monitor Scale Factor ////////////////////////////////////////////
        u_monitor_scale = new monitor_scale("u_monitor_scale");
        u_monitor_scale->preflag_out(preflag_out);
        u_monitor_scale->part2_start(part2_start);
#endif
    }
};

// MP3 Decoder Instance /////////////////////////////////////////
extern mp3_decoder  u_mp3_decoder;

#endif
