///////////////////////////////////////////////////////////////////////////
// Filename : Mp3Stream.hxx
// Project  : Ans_MP3_Player/MP3_Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision : 5-4-01: Modified for interface to untimed SystemC testbench
//                    - John Stickley, Ikos Systems, Inc. (johns@ikos.com)
// Comment  :
//
//

//===========================================================================
// Mp3Stream                                                    johnS 4-21-01
//===========================================================================
 
#include <stdio.h>
#include <string>
#include <systemc.h>

#include "Mp3Common.h"

SC_MODULE( Mp3Stream ){

    //-------------------------------------------------------
    // Abstract port declarations
    sc_slave<>                          RequestFrameIn;
    sc_outslave<const Mp3Header *>      FrameHeaderOut;
    sc_outslave<const Mp3DataBuffer *>  DataBufferOut;
    sc_outmaster<const PcmFileInfo *>   FileHeaderOut;
    sc_master<>                         EosOut;

    //-------------------------------------------------------
    // Context declarations
    private:
        FILE *dIfp;
        Mp3Header dMp3Header;

        Mp3DataBuffer dInBuffer; // MP3 Stream Buffer
        unsigned long dInBufferTotalBits;
        int dInBufferBitIndex;

        Mp3DataBuffer dOutBuffer;
        int dFirstConsoleDisplay;
        string dPcmFileName;
 
    //-------------------------------------------------------
    // Thread declarations
    private:
        void requestFrameInThread();

        // Send header, data to decoder FrameBuffer ...
        void frameHeaderOutThread();
        void dataBufferOutThread() { DataBufferOut  = &dOutBuffer; }

    //-------------------------------------------------------
    // Accessors
    public:
        void Open( const char *inputFileName ){
            dPcmFileName = inputFileName;

            if( (dIfp=fopen(inputFileName, "r+b")) == NULL )
                errorFileOpen( inputFileName, "reading" );
            }
 
    //-------------------------------------------------------
    // Helper declarations
    private:
        int fillStreamBuffer();
        void displayToConsole();
        Mp3FrameHeader seekHeader();
        void getSideHeaderBlock(Mp3SideHeaderBlock *shb);
        int getMainDataSlots();

        // Request more bits from MP3 stream buffer
        unsigned long getBits( int range ) {

            register unsigned long data = 0;
            register unsigned char ch = dInBuffer.Buffer[dInBuffer.ReadIndex];

            for( int i=0; i<range; i++ ){
                data <<= 1;
                data |= (ch >> dInBufferBitIndex) & 1;

                // if( current byte is exhausted ) ...
                if( --dInBufferBitIndex < 0 ){
                    // if( check for wraparound of circular buffer ) ...
                    if( ++dInBuffer.ReadIndex == FRAME_SIZE )
                        dInBuffer.ReadIndex = 0;
                    ch = dInBuffer.Buffer[dInBuffer.ReadIndex];
                    dInBufferBitIndex = 7;
                    }
                }
            dInBufferTotalBits += range;
            return data;
            }

        void putOutBuffer( unsigned char byte ){
            dOutBuffer.Buffer[dOutBuffer.WriteIndex++] = byte; }

        static void errorFileOpen(
                const string &fileName, const char *forWhat ){
            string message( "Cannot open file " );
            message += fileName; message += " for "; message += forWhat;
            throw message;
            }

        static void errorSideInfo(){
            string message(
                "Side info bad: block_type == 0 in split block.\n" );
            throw message;
            }

    public:
        SC_CTOR( Mp3Stream )
            : dIfp(NULL), dInBufferTotalBits(0), dInBufferBitIndex(7),
              dFirstConsoleDisplay(1)

        {   //--------------------------------------
            // Thread bindings
            dInBuffer.ReadIndex = dInBuffer.WriteIndex = 0;
            dOutBuffer.ReadIndex = dOutBuffer.WriteIndex = 0;

            SC_SLAVE( requestFrameInThread, RequestFrameIn );
            SC_SLAVE( frameHeaderOutThread, FrameHeaderOut );
            SC_SLAVE( dataBufferOutThread,  DataBufferOut );
            }
    };
