///////////////////////////////////////////////////////////////////////////
// Filename : PcmStream.cx
// Project  : Ans_MP3_Player/MP3_Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision : 5-4-01: Modified for interface to untimed SystemC testbench
//                    - John Stickley, Ikos Systems, Inc. (johns@ikos.com)
// Comment  :
//
 
#include <stdio.h>
#include <string>
#include "PcmStream.hxx"
#include "WavImage.h"

//===========================================================================
// PcmStream                                                    johnS 4-21-01
//===========================================================================
 
//---------------------------------------------------------
// ::fileHeaderInThread() - 1-time acceptance of a representative
//                          MP3 frame header
//
// Description                                johnS 4-21-01
// -----------
// This routine accepts a representative frame header 1 time only from
// the hardware.  This MP3 frame header is then used to derive any infomation
// that needs to go into the output file header.
//
// A small synopsis of the header information is also printed to the screen.
//---------------------------------------------------------

void PcmStream::fileHeaderInThread(){
    const PcmFileInfo *pcmFileInfo = FileHeaderIn;

    dIsStereo = pcmFileInfo->Stereo;
 
    if( !dWavImage ) {

        dWavImage = new WavImage();
        dWavImage->Setup(
            pcmFileInfo->Stereo+1,            // # channels
            16,                               // bits/sample
            0,                                // # samples
            pcmFileInfo->SamplingFrequency ); // sample rate

        dWavImage->Save( pcmFileInfo->FileName );
        }
}
 
//---------------------------------------------------------
// ::pcmFrameInThread() - Accept next output PCM stream
//                        buffer from the hardware.
//
// Description                                johnS 4-21-01
// -----------
// This is an 'inslave' thread that accepts the next frame
// of decoded PCM samples on demand from the subband
// synthesis stage of the decoder.  These samples are
// channeled to the supported PCM file format.
//---------------------------------------------------------

extern "C" {
extern unsigned long updateCrc( int ch, unsigned long CRC );
};

void PcmStream::pcmFrameInThread(){
    register int ss, sb, ch, stereo = dIsStereo;
    register short pcm;
    dSampleFrames++;

    const PcmSampleFrame *pcmFrame = PcmFrameIn;
//unsigned long crc = 0xffffffffL;
 
    for ( ss=0; ss<SSLIMIT; ss++)
        for ( sb=0; sb<SBLIMIT; sb++)
            for ( ch=0; ch<=stereo; ch++) {
                dWavImage->Write( (pcm=pcmFrame->Data[ch][ss][sb]) );
                dPcmCount++;
//crc = updateCrc( pcm, crc );
                if (pcm > 14900)
                    dPcmOverflowCount++;
                }

#if 0
FILE *fp;
fp = fopen( "pcmcrc.log", "a+" );
fprintf(fp, "Frame #%d CRC: %x\n", dSampleFrames, crc );
fflush(fp);
fclose(fp);
#endif
    }
 
//---------------------------------------------------------
// ::doneInThread() - Close output file, print stats.
//
// Description                                johnS 4-21-01
// -----------
// This thread is done at the completion of the MP3
// stream conversion.
//---------------------------------------------------------

void PcmStream::doneInThread(){
    cout << "Total sample frames: " << dSampleFrames << endl;
    cout << "Total PCM sample count: " << dPcmCount << endl;
    cout << "Total PCM samples whose values overflowed 14900: "
         << dPcmOverflowCount << endl;
    delete dWavImage;
    }
