///////////////////////////////////////////////////////////////////////////
// Filename : Top.cxx
// Project  : Ans_MP3_Player/MP3_Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision : 5-4-01: Modified for interface to untimed SystemC testbench
//                    - John Stickley, Ikos Systems, Inc. (johns@ikos.com)
// Comment  :
//
//

#include <string>
#include <systemc.h>

//#include <SceMi.hxx>
#include "Mp3Common.h"
 
sc_clock UTick;
    // Global "Untimed Tick" clock (not an oxymoron ! :-)
    //    This is used for atomic advance of autonomous threads since
    //    there seems to be no implicitly defined way to do this in SystemC
    //    as there is in CoWare.
    //   -- johnS 6-22-00

sc_clock clock( "clock", CLOCK_PERIOD );
    // This the clock for all timed functional modules.

#include "PlayControl.hxx"
#include "Mp3Stream.hxx"
#include "Mp3Decoder.hxx"
#include "PcmStream.hxx"

#if 0
#include "SceMiDispatcher.hxx"

static void errorHandler( void */*context*/, SceMiEC *ec  );
    // Forward declaration (see end of file).
#endif

SC_MODULE( Top ){

    sc_link_mp<> play, stop, done, eos, closePcm;
    sc_link_mp<>                       requestFrame;
    sc_link_mp<const Mp3Header *>      mp3FrameHeader;
    sc_link_mp<const Mp3DataBuffer *>  mp3DataBuffer;
    sc_link_mp<const PcmSampleFrame *> pcmSampleFrame;
    sc_link_mp<const PcmFileInfo *>    pcmFileInfo;

    //-------------------------------------------------------
    // Module declarations
    PlayControl *playControl;
    Mp3Stream   *mp3Stream;
    Mp3Decoder  *mp3Decoder;
    PcmStream   *pcmStream;

    //SceMiDispatcher *sceMiDispatcher;
 
    SC_CTOR( Top ){

        playControl = new PlayControl( "playControl" );
            playControl->PlayOut( play );                      // --> master
            playControl->StopOut( stop );                      // --> master
            playControl->EosIn( eos );                         // <-- slave
            playControl->DoneIn( done );                       // <-- slave
            playControl->ClosePcm( closePcm );                 // <-- master

        mp3Stream = new Mp3Stream( "mp3Stream" );
            mp3Stream->RequestFrameIn( requestFrame );         // <-- slave
            mp3Stream->FrameHeaderOut( mp3FrameHeader );       // --> outslave
            mp3Stream->DataBufferOut( mp3DataBuffer );         // --> outslave
            mp3Stream->FileHeaderOut( pcmFileInfo );           // --> outmaster
            mp3Stream->EosOut( eos );                          // <-- master

        mp3Decoder = new Mp3Decoder( "mp3Decoder" );
            mp3Decoder->PlayIn( play );                        // <-- slave
            mp3Decoder->StopIn( stop );                        // <-- slave
            mp3Decoder->DoneOut( done );                       // --> master
            mp3Decoder->RequestFrameOut( requestFrame );       // --> master
            mp3Decoder->FrameHeaderIn( mp3FrameHeader );       // <-- inmaster
            mp3Decoder->DataBufferIn( mp3DataBuffer );         // <-- inmaster
            mp3Decoder->PcmFrameOut( pcmSampleFrame );         // --> outmaster

        pcmStream = new PcmStream( "pcmStream" );
            pcmStream->FileHeaderIn( pcmFileInfo );            // <-- inslave
            pcmStream->PcmFrameIn( pcmSampleFrame );           // <-- inslave
            pcmStream->DoneIn( closePcm );                     // <-- slave

        //sceMiDispatcher = new SceMiDispatcher( "sceMiDispatcher" );
        //  sceMiDispatcher->DoneIn( done );                   // <-- slave
        }
    };

int sc_main( int argc, char *argv[] ){
 
    const char *mp3FileName;
    string pcmFileName;
    if( argc != 2 ){
        cout << "Usage: Mp3Decoder <filename.mp3>" << endl;
        exit(-1);
        }
    mp3FileName = argv[1];

    //---------------------------------------------------------
    // Instantiate SceMi

    //SceMi::RegisterErrorHandler( errorHandler, NULL );
    //SceMi *sceMi = NULL;

    try {
        //int sceMiVersion = SceMi::Version( SCEMI_VERSION_STRING );
        //SceMiParameters parameters( "./params.mct" );
        //sceMi = SceMi::Init( sceMiVersion, &parameters );

        //---------------------------------------------------------
        // Instantiate the system here.  Autonomous threads nested
        // inside the DispatcherDriver and the Testbench will advance
        // untimed activity.  Such threads are senstitive to UTick defined
        // at the top of this file.
        // -- johnS  8-29-00
     
        Top top( "top" );

        //---------------------------------------------------------
        // Open the MP3 Input File
        // Establish proper bindings between the SCEMI and the modules
        // modules that directly interact with it.

        top.mp3Stream->Open( mp3FileName );
        top.pcmStream->SetFileBaseName( mp3FileName );
        //top.sceMiDispatcher-> Bind( sceMi );

        //-------------------------------------------------
        // Kick off SystemC kernel ...
        cerr << "Let 'er rip !" << endl;
        sc_start(-1);
        }

    catch( string message ) {
        cerr << message << endl;
        cerr << "Fatal Error: Program aborting." << endl;
        //if( sceMi ) SceMi::Shutdown( sceMi );
        return -1;
        }
    catch(...) {
        cerr << "Error: Unclassified exception." << endl;
        cerr << "Fatal Error: Program aborting." << endl;
        //if( sceMi ) SceMi::Shutdown( sceMi );
        return -1;
        }
    return 0;
}

#if 0
static void errorHandler( void */*context*/, SceMiEC *ec  ) {
    char buf[32];

    sprintf( buf, "%d", (int)ec->Type );
    string messageText( "SCEMI Error[" );
    messageText += buf;
    messageText += "]: Function: ";
    messageText += ec->Culprit;
    messageText += "\n";
    messageText += ec->Message;
    throw messageText;
}
#endif
