#ifndef _wavimage_h
#define _wavimage_h

#include <string>
#include <fstream>

using namespace std;

//--------------------------------------------------------------------------
//  WavFile Image
//

class WavIO;

class WavImage {
    protected:

        string dFileName;
        std::ifstream *dIfs;
        std::ofstream *dOfs;

        unsigned long  dFileLength;     // Length of whole file - 8

        unsigned long  dFmtChunkLength; // Length of fmt chunk.
        unsigned short dFmtTag;         // 1 denotes PCM format.
        unsigned short dFmtChannels;    // 1 denotes mono, 2 denotes stereo.
        unsigned long  dFmtSamplesPerSec; // Sample Rate.
        unsigned long  dFmtBytesPerSec; // SamplesPerSecond * BlockAlign
        unsigned short dFmtBlockAlign;  // Channels * BitsPerSample / 8
        unsigned short dFmtBitsPerSample;

        unsigned long dDataLength;      // Length of data section.
        unsigned long dNumSamples;      // Number of samples.
        unsigned long dTimeSpan;        // Time span.
        unsigned long dActualNumSamples;

    private:
        WavIO *dWavIO;      // Machine specific I/O routine tray
        void classifyIO();

    public:
        WavImage() : dIfs(NULL), dOfs(NULL), dActualNumSamples(0) {}
        ~WavImage();

        void Setup(
            unsigned short channels, unsigned short bitsPerSample,
            unsigned long numSamples, unsigned long samplesPerSec );

        void Load( const char *fileName );  // Load wav file image;
        void Save( const char *fileName );  // Save wav file image;

#ifdef WIN32
        std::ifstream &InStream() { return *dIfs; }
#else
        ifstream &InStream() { return *dIfs; }
#endif
        void Write( unsigned short data );
        void Write( unsigned long data );
        void Write( short data );
        void Write( long data );

        void Read( unsigned char &data );
        void Read( unsigned short &data );
        void Read( unsigned long &data );
        void Read( char &data );
        void Read( short &data );
        void Read( long &data );
    };
#endif
