///////////////////////////////////////////////////////////////////////////
// Filename : sc_III_Antialias.cpp
// Project  : Ans_MP3_Player/MP3_Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision : 5-4-01: Modified for interface to untimed SystemC testbench
//                    - John Stickley, Ikos Systems, Inc. (johns@ikos.com)
// Comment  :
//
//

#include "sc_III_Antialias.h"

#include "sc_III_Decoder_Mem.h"

void iii_antialias::proc_antialias() {

    if (rst.read()==(sc_bit)'0')    // Async. Reset
    {
        Antialias_State = S_ANTIALIAS_WAIT;
        ready = (sc_bit)'1';
    }
    else if (clk.event() && clk.read()==(sc_bit)'1')    // Clocking Rising-Edge
    {
        switch(Antialias_State.read())
        {
            case S_ANTIALIAS_WAIT:
                if (start.read()==(sc_bit)'1')
                {
                    ready = (sc_bit)'0';
                    Antialias_State = S_ANTIALIAS_DECODE;
                }
                break;

            case S_ANTIALIAS_DECODE:
                if (start.read()==(sc_bit)'0') {
                    const Mp3SideHeader *sideHeader = SideHeader.read();

                    int t_ch = channel.read();

                    III_antialias(
                        sideHeader->WindowSwitchingFlag,
                        sideHeader->BlockType,
                        sideHeader->MixedBlockFlag,
                        t_ch );

                    Antialias_State = S_ANTIALIAS_DOWNLOAD;
                }
                break;

            case S_ANTIALIAS_DOWNLOAD:
                Antialias_State = S_ANTIALIAS_BUSY;
                break;

            case S_ANTIALIAS_BUSY:
                ready = (sc_bit)'1';
                Antialias_State = S_ANTIALIAS_WAIT;
                break;

            default:
                Antialias_State = S_ANTIALIAS_WAIT;
                break;
        }
    }
}

///////////////////////////////////////////////////////////////////////////
// Antialias
//

void iii_antialias::III_antialias(
    int window_switching_flag,
    int block_type,
    int mixed_block_flag,
    int ch )
{
    double  bu, bd;  // upper and lower butterfly inputs
    int     ss, sb, sblim;

    // clear all inputs

    if (window_switching_flag && (block_type == 2) && !mixed_block_flag )
        return;

    if (window_switching_flag && (block_type == 2) && mixed_block_flag )
        sblim = 1;
    else
        sblim = SBLIMIT-1;

    // 31 alias-reduction operations between each pair of sub-bands
    // with 8 butterflies between each pair

    for(sb=0;sb<sblim;sb++)
        for(ss=0;ss<8;ss++)
        {
            bu = GlobalDecoderMem->get_re( ch, sb, 17-ss);
            bd = GlobalDecoderMem->get_re( ch, sb+1, ss);
            GlobalDecoderMem->put_hybridIn(
                    ((bu * cs[ss]) - (bd * ca[ss])), ch, sb, (17-ss) );
            GlobalDecoderMem->put_hybridIn(
                    ((bd * cs[ss]) + (bu * ca[ss])), ch, (sb+1), ss);
        }  
}
