///////////////////////////////////////////////////////////////////////////
// Filename : sc_III_Decoder_Mem.h
// Project  : Ans_MP3_Player/MP3_Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision : 5-4-01: Modified for interface to untimed SystemC testbench
//                    - John Stickley, Ikos Systems, Inc. (johns@ikos.com)
// Comment  :
//

#ifndef DECODER_MEM_INC
#define DECODER_MEM_INC

#include <systemc.h>
#include "Mp3Common.h"

enum S_Mem_Init_Type {
    S_MEM_INIT_WAIT,
    S_MEM_INIT_BUSY,
    S_MEM_INIT_DONE
};

SC_MODULE(decoder_mem)
{
    sc_in_clk           clk;                // Clock
    sc_in<sc_bit>       rst;                // Reset

    sc_in<sc_bit>       mem_init;           // Frame Memory Initialize
    sc_out<sc_bit>      m_init_done;

    // Process ///////////////////////////////////////////////////////////
    void proc_mem_init();

    // Signals ///////////////////////////////////////////////////////////
    sc_signal<S_Mem_Init_Type>  Mem_Init_State;

    // Global Memories ///////////////////////////////////////////////////
    III_scalefac_t  scalefac;                           // Scale Factor

    double          ram[2][SBLIMIT][SSLIMIT];           // Decoder Memory
    double          prevblck[2][SBLIMIT][SSLIMIT];      // Hybrid result of previous Frame/Granule
    double          rawout[36];                         // Temp.

    // Member functions //////////////////////////////////////////////////
    public:
        // Read/Write Scale factor data
        long get_scalefac(
                int ch, int window, int cb, int block_type){
            if (block_type==0)
                return(scalefac[ch].l[cb]);
            else
                return(scalefac[ch].s[window][cb]);
            }
        void put_scalefac(
                long int data, int ch, int window, int cb, int block_type){
            if (block_type==0)
                scalefac[ch].l[cb] = data;
            else
                scalefac[ch].s[window][cb] = data;
            }
        // Read/Write Huffman decoded
        long    get_is(int ch, int sb, int ss){
                    return((long)ram[ch][sb][ss]); }
        void    put_is(long data, int ch, int sb, int ss){
                    ram[ch][sb][ss] = (double)data; }
        // Read/Write Requantized data
        double  get_ro(int ch, int sb, int ss){
                    return(ram[ch][sb][ss]); }
        void    put_ro(double data, int ch, int sb, int ss){
                    ram[ch][sb][ss] = data; }
        // Read/Write Stero processed data
        double  get_lr(int ch, int sb, int ss){ return(ram[ch][sb][ss]); }
        void    put_lr(double data, int ch, int sb, int ss){
                    ram[ch][sb][ss] = data; }
        // Read/Write Reordered data
        double  get_re(int ch, int sb, int ss){ return(ram[ch][sb][ss]); }
        void    put_re(double data, int ch, int sb, int ss){
                    ram[ch][sb][ss] = data; }
        // Read/Write Antialiased data
        double  get_hybridIn(int ch, int sb, int ss){
                    return(ram[ch][sb][ss]); }
        void    put_hybridIn(double data, int ch, int sb, int ss){
                    ram[ch][sb][ss] = data; }
        // Read/Write Hybrid
        double  get_hybridOut(int ch, int sb, int ss){
                    return(ram[ch][sb][ss]); }
        void    put_hybridOut(double data, int ch, int sb, int ss){
                    ram[ch][sb][ss] = data; }
        // Read/Write Hybrid-Prev
        double  get_prevblck(int ch, int sb, int ss){
                    return(prevblck[ch][sb][ss]); }
        void    put_prevblck(double data, int ch, int sb, int ss){
                    prevblck[ch][sb][ss] = data; };
        // Read/Write Hybrid-Rawout
        double  get_rawout(int ss){ return(rawout[ss]); }
        void    put_rawout(double data, int ss){ rawout[ss] = data; }
        // Read/Write PCM Data

    // Constructor ///////////////////////////////////////////////////////////
    SC_CTOR(decoder_mem)
    {
        SC_METHOD(proc_mem_init);
        sensitive << clk << rst;
    }
};

extern decoder_mem *GlobalDecoderMem;

#endif
