///////////////////////////////////////////////////////////////////////////
// Filename : sc_III_Dequantizer.cpp
// Project  : Ans_MP3_Player/MP3_Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision : 5-4-01: Modified for interface to untimed SystemC testbench
//                    - John Stickley, Ikos Systems, Inc. (johns@ikos.com)
// Comment  :
//
//

#include "math.h"
#include "sc_III_Dequantizer.h"
#include "sfBandIndex.h"

#include "sc_III_Decoder_Mem.h"

void iii_dequantizer::proc_deq() {
    if (rst.read()==(sc_bit)'0')    // Async. Reset
    {
        Deq_State = S_DEQ_WAIT;
        ready = (sc_bit)'1';
    }
    else if (clk.event() && clk.read()==(sc_bit)'1')    // Clocking Rising-Edge
    {
        switch(Deq_State.read())
        {
            case S_DEQ_WAIT:
                if (start.read()==(sc_bit)'1')
                {
                    ready = (sc_bit)'0';
                    Deq_State = S_DEQ_DECODE;
                }
                break;

            case S_DEQ_DECODE:
                if (start.read()==(sc_bit)'0') {
                    Mp3FrameHeader frameHeader      = FrameHeader.read();
                    const Mp3SideHeader *sideHeader = SideHeader.read();

                    int t_preflag = preflag.read();
                    int t_channel = channel.read();

                    III_dequantize_sample(
                        (int)Mp3HeaderSamplingFrequency(frameHeader),
                        (int)Mp3HeaderVersion(frameHeader),
                        (int)sideHeader->WindowSwitchingFlag,
                        (int)sideHeader->BlockType,
                        (int)sideHeader->MixedBlockFlag,
                        (int)sideHeader->GlobalGain,
                        (int *)sideHeader->SubblockGain,
                        (int)sideHeader->ScalefacScale,
                        t_preflag,
                        t_channel);

                    Deq_State = S_DEQ_DOWNLOAD;
                }
                break;

            case S_DEQ_DOWNLOAD:
                Deq_State = S_DEQ_BUSY;
                break;

            case S_DEQ_BUSY:
                ready = (sc_bit)'1';
                Deq_State = S_DEQ_WAIT;
                break;

            default:
                Deq_State = S_DEQ_WAIT;
                break;
        }
    }
}

//////////////////////////////////////////////////////////////////////////
// Dequantizer for Layer III
//
int pretab[22] = {0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,2,2,3,3,3,2,0};

void iii_dequantizer::III_dequantize_sample(
    int sampling_frequency,
    int version,
    int window_switching_flag,
    int block_type,
    int mixed_block_flag,
    int global_gain,
    int subblock_gain[3],
    int scalefac_scale,
    int preflag,
    int ch)
{
    int ss,sb,cb=0,sfreq;
    int next_cb_boundary, cb_begin, cb_width, sign;

    double  xr, xrinit;
    xrinit = pow( 2.0 , (0.25 * (global_gain - 210.0)));

    sfreq = sampling_frequency + (version * 3);

    // choose correct scalefactor band per block type, initalize boundary 
    if (window_switching_flag && (block_type == 2) )
        if (mixed_block_flag) 
            next_cb_boundary=sfBandIndex[sfreq].l[1];  // LONG blocks
        else
        {
            next_cb_boundary = sfBandIndex[sfreq].s[1]*3; // pure SHORT block
            cb_width = sfBandIndex[sfreq].s[1];
            cb_begin = 0;
        }
    else 
        next_cb_boundary = sfBandIndex[sfreq].l[1];  // LONG blocks

    // apply formula per block type
    for (sb=0 ; sb < SBLIMIT ; sb++)
    {
        for (ss=0 ; ss < SSLIMIT ; ss++)
        {
            if ( (sb*18)+ss == next_cb_boundary)    // Adjust critical band boundary
            {
                if (window_switching_flag && (block_type == 2))
                {
                    if (mixed_block_flag)
                    {
                        if (((sb*18)+ss) == sfBandIndex[sfreq].l[8])
                        {
                            next_cb_boundary = sfBandIndex[sfreq].s[4]*3; 
                            cb = 3;
                            cb_width = sfBandIndex[sfreq].s[cb+1] - sfBandIndex[sfreq].s[cb];
                            cb_begin = sfBandIndex[sfreq].s[cb]*3;
                        }
                        else if (((sb*18)+ss) < sfBandIndex[sfreq].l[8]) 
                            next_cb_boundary = sfBandIndex[sfreq].l[(++cb)+1];
                        else
                        {
                            next_cb_boundary = sfBandIndex[sfreq].s[(++cb)+1]*3;
                            cb_width = sfBandIndex[sfreq].s[cb+1] - sfBandIndex[sfreq].s[cb];
                            cb_begin = sfBandIndex[sfreq].s[cb]*3;      
                        }   
                    }
                    else
                    {
                        next_cb_boundary = sfBandIndex[sfreq].s[(++cb)+1]*3;
                        cb_width = sfBandIndex[sfreq].s[cb+1] - sfBandIndex[sfreq].s[cb];
                        cb_begin = sfBandIndex[sfreq].s[cb]*3;
                    } 
                }
                else // Long blocks
                    next_cb_boundary = sfBandIndex[sfreq].l[(++cb)+1];
            }

            // Compute overall (global) scaling.
//          xr = pow( 2.0 , (0.25 * (global_gain - 210.0)));
            xr = xrinit;

            // Do long/short dependent scaling operations.
            if (window_switching_flag && (
                    ((block_type == 2) && (mixed_block_flag == 0)) ||
                    ((block_type == 2) && mixed_block_flag && (sb >= 2)) ))
            {
                xr *= pow(2.0, 0.25 * -8.0 * subblock_gain[(((sb*18)+ss) - cb_begin)/cb_width]);
                xr *= pow(2.0, 0.25 * -2.0 * (1.0+scalefac_scale)
                                    * GlobalDecoderMem->get_scalefac(
                                            ch, (((sb*18)+ss) - cb_begin)/cb_width, cb, 1) );
            }
            else    // LONG block types & 1st 2 subbands of switched blocks
            {   
                xr *= pow(2.0, -0.5 * (1.0+scalefac_scale)
                                    * ( GlobalDecoderMem->get_scalefac(ch, 0, cb, 0)
                                        + preflag*pretab[cb]) );
            }
            long get_is = GlobalDecoderMem->get_is( ch, sb, ss);
            // Scale quantized value.
            sign = (get_is < 0) ? 1 : 0;
            xr *= pow( (double) abs(get_is), ((double)4.0/3.0) );
            if (sign)
                xr = -xr;

            // Finally Write dequantized value
            GlobalDecoderMem->put_ro( xr, ch, sb, ss);
        }
    }
}
