///////////////////////////////////////////////////////////////////////////
// Filename : sc_III_Dequantizer.h
// Project  : Ans_MP3_Player/MP3_Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision : 5-4-01: Modified for interface to untimed SystemC testbench
//                    - John Stickley, Ikos Systems, Inc. (johns@ikos.com)
// Comment  :
//
//

#ifndef III_DEQUANTIZER_INC
#define III_DEQUANTIZER_INC

#include <systemc.h>
#include "Mp3Common.h"

enum tDeq_State {
    S_DEQ_WAIT,
    S_DEQ_DECODE,
    S_DEQ_DOWNLOAD,
    S_DEQ_BUSY };

SC_MODULE(iii_dequantizer)
{
    sc_in_clk           clk;    // Clock
    sc_in<sc_bit>       rst;    // Reset

    sc_in<sc_bit>       start;  // Dequantizer Start
    sc_out<sc_bit>      ready;  // Dequantizer Ready

    sc_in<sc_bit>       channel;

    sc_in<sc_bit>       preflag;
 
    sc_in<Mp3FrameHeader>         FrameHeader;
    sc_in<const Mp3SideHeader *>  SideHeader;
    
    // Process ///////////////////////////////////////////////////////////
    void proc_deq();

    void III_dequantize_sample(
        int sampling_frequency,
        int version,
        int window_switching_flag,
        int block_type,
        int mixed_block_flag,
        int global_gain,
        int subblock_gain[3],
        int scalefac_scale,
        int preflag,
        int ch);

    // Signals ///////////////////////////////////////////////////////////
    sc_signal<tDeq_State>   Deq_State;  // FSM State Variable

    // Constructor ///////////////////////////////////////////////////////////
    SC_CTOR(iii_dequantizer)
    {
        SC_METHOD(proc_deq);
        sensitive << clk << rst;
    }
};

#endif
