///////////////////////////////////////////////////////////////////////////
// Filename : sc_III_Hybrid.h
// Project  : Ans_MP3_Player/MP3_Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision : 5-4-01: Modified for interface to untimed SystemC testbench
//                    - John Stickley, Ikos Systems, Inc. (johns@ikos.com)
// Comment  :
//
//

#ifndef III_HYBRID_INC
#define III_HYBRID_INC

#include <systemc.h>
#include "Mp3Common.h"

enum tHybrid_State {
    S_HYBRID_WAIT,
    S_HYBRID_DECODE,
    S_HYBRID_DOWNLOAD,
    S_HYBRID_BUSY };

SC_MODULE(iii_hybrid)
{
    sc_in_clk           clk;    // Clock
    sc_in<sc_bit>       rst;    // Reset

    sc_in<sc_bit>       start;  // Hybrid Start
    sc_out<sc_bit>      ready;  // Hybrid Ready

    sc_in<sc_bit>       channel;
 
    sc_in<Mp3FrameHeader>         FrameHeader;
    sc_in<const Mp3SideHeader *>  SideHeader;

    // Process ///////////////////////////////////////////////////////////
    void proc_hybrid();

    // Global Signals ////////////////////////////////////////////////////
    sc_signal<tHybrid_State>    Hybrid_State;   // FSM State Variable

    // Functions /////////////////////////////////////////////////////////
    void III_hybrid(
        int     window_switching_flag,
        int     mixed_block_flag,
        int     block_type,
        int     ch );

    void inv_mdct(int ch, int sb, int block_type);
    void create_imdct_coeff();

    double  win[4][36];
    double  COS36[4*36];
    double  COS12[6*12];

    // Constructor ///////////////////////////////////////////////////////////
    SC_CTOR(iii_hybrid)
    {
        SC_METHOD(proc_hybrid);
        sensitive << clk << rst;

        // Create COS & WIN Coefficients
        create_imdct_coeff();
    }
};

#endif
